#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
from ._ost_gfx import *
from .py_gfx_obj import PyGfxObj
import functools

WHITE=RGB(1.0,1.0,1.0)
BLACK=RGB(0.0,0.0,0.0)
GREY=RGB(0.5,0.5,0.5)
RED=RGB(1.0,0.0,0.0)
DARKRED=RGB(0.5,0.0,0.0)
LIGHTRED=RGB(1.0,0.5,0.5)
GREEN=RGB(0.0,1.0,0.0)
DARKGREEN=RGB(0.0,0.5,0.0)
LIGHTGREEN=RGB(0.5,1.0,0.5)
BLUE=RGB(0.0,0.0,1.0)
DARKBLUE=RGB(0.0,0.0,0.5)
LIGHTBLUE=RGB(0.5,0.5,1.0)
YELLOW=RGB(1.0,1.0,0.0)
DARKYELLOW=RGB(0.5,0.5,0.0)
LIGHTYELLOW=RGB(1.0,1.0,0.5)
CYAN=RGB(0.0,1.0,1.0)
DARKCYAN=RGB(0.0,0.5,0.5)
LIGHTCYAN=RGB(0.5,1.0,1.0)
MAGENTA=RGB(1.0,0.0,1.0)
DARKMAGENTA=RGB(0.5,0.0,0.5)
LIGHTMAGENTA=RGB(1.0,0.5,1.0)
PURPLE=MAGENTA
DARKPURPLE=DARKMAGENTA
LIGHTPURPLE=LIGHTMAGENTA
ORANGE=RGB(1.0,0.5,0.0)
DARKORANGE=RGB(0.5,0.25,0.0)
LIGHTORANGE=RGB(1.0,0.75,0.5)

def Stereo(mode,flip=None,alg=None):
  """
  Stereo control

  :param mode: 0=off, 1=quad-buffered, 2=interlaced
  :type mode: int
  :param flip: invert order of left/right display
  :type flip: bool
  :param alg: stereo algorithm (0 or 1)
  :type param: int
  """
  if(flip):
    Scene().SetStereoFlip(flip)
  if(alg):
    Scene().SetStereoAlg(alg)

  Scene().SetStereoMode(mode)

def FitToScreen(gfx_ent, width=None, height=None, margin=0.05):
  """
  Setup camera such that it is centered on the graphical entity and the entity 
  fits the entire viewport. The longest axes of the entity are aligned along 
  the x- and y- axes of the screen.
  
  :param gfx_ent: The graphical entity
  :type gfx_ent: str or :class:`Entity`
  
  
  """
  from ost import geom
  import math
  def _XYZ(view, axes):
    """
    returns the vectors in x, y and z direction respectively. The smallest 
    vector is in z, then y, and the largest along x.
    """
    rows=[axes.GetRow(i) for i in range(3)]
    lengths=[]
    for axe in rows:
      min_proj=geom.Dot(axe, view.atoms[0].pos)
      max_proj=min_proj
      for atom in view.atoms[1:]:
        proj=geom.Dot(axe, atom.pos)
        min_proj=min(proj, min_proj)
        max_proj=max(proj, max_proj)
      lengths.append(max_proj-min_proj)
    def cmp_x(rhs, lhs):
      # replaced cmp when porting to Python 3
      #return cmp(lhs[1], rhs[1])
      return (lhs[1] > rhs[1]) - (lhs[1] < rhs[1])
    sorted_axes=sorted(zip(rows, lengths), key=functools.cmp_to_key(cmp_x))
    return [r*l for r,l in sorted_axes]
  scene=Scene()
  if not isinstance(gfx_ent, Entity):
    gfx_ent=scene[str(gfx_ent)]
  width=width and width or scene.viewport.width
  height=height and height or scene.viewport.height
  atom_positions=geom.Vec3List([atom.pos for atom in gfx_ent.view.atoms])
  axes=atom_positions.principal_axes
  sorted_axes=_XYZ(gfx_ent.view, axes)
  x_bigger_than_y=geom.Length(sorted_axes[0])>geom.Length(sorted_axes[1])
  if x_bigger_than_y:
    if width>height:    
      x_axes=geom.Normalize(sorted_axes[0])
      y_axes=geom.Normalize(sorted_axes[1])
    else:
      x_axes=geom.Normalize(sorted_axes[1])
      y_axes=geom.Normalize(sorted_axes[0])
  else:
    if width>height:    
      x_axes=geom.Normalize(sorted_axes[1])
      y_axes=geom.Normalize(sorted_axes[0])
    else:
      x_axes=geom.Normalize(sorted_axes[0])
      y_axes=geom.Normalize(sorted_axes[1])
  z_axes=geom.Normalize(geom.Cross(x_axes, y_axes))
  rotation=geom.Mat3(x_axes[0], x_axes[1], x_axes[2],
                     y_axes[0], y_axes[1], y_axes[2],
                     z_axes[0], z_axes[1], z_axes[2])
  rtc=geom.Mat4(rotation)
  
  center=gfx_ent.center
  aspect=float(width)/float(height)
  factor_y=1.0/math.tan(math.radians(scene.fov))
  factor_x=factor_y/aspect
  z_off=geom.Length(sorted_axes[2])*0.5
  rtc[0,3]=center[0]
  rtc[1,3]=center[1]
  rtc[2,3]=center[2]
  rtc[3,0]=0
  rtc[3,1]=0
  rtc[3,2]=-(max(factor_x*(1+margin)*geom.Length(sorted_axes[0]), 
                factor_y*(1+margin)*geom.Length(sorted_axes[1]))+z_off)
  scene.SetRTC(rtc)


class GfxNodeListAttrProxy:
  def __init__(self, node_list, name):
    self._node_list=node_list
    self._name=name

  def __iter__(self):
    for node in self._node_list:
      yield getattr(node, self._name)

  def __call__(self, *args, **kwargs):
    for node in self._node_list:
      bound_method=getattr(node, self._name)
      bound_method(*args, **kwargs)

class GfxNodeListProxy(object):
  def __init__(self, node_list):
    self._nodes=node_list

  def __getattr__(self, name):
    if name.startswith('_'):
      return super(GfxNodeListProxy, self).__getattr__(name)
    return GfxNodeListAttrProxy(self._nodes, name)

  def __setattr__(self, name, value):
    if name.startswith('_'):
      super(GfxNodeListProxy, self).__setattr__(name, value)
    for node in self._nodes:
      setattr(node, name, value)

def _Match(scene, pattern="*"):
  import os
  import fnmatch
  def _Recurse(path, node, pattern):
    matches=[]
    for child in node.children:
      full_name=os.path.join(path, child.name)
      if fnmatch.fnmatchcase(full_name, pattern):
        matches.append(child)
      matches.extend(_Recurse(full_name, child, pattern))
    return matches
  return GfxNodeListProxy(_Recurse("", Scene().root_node, pattern))

SceneSingleton.Match=_Match

def _to_vec3(p):
  import ost.geom
  if isinstance(p,ost.geom.Vec3):
    return p
  else:
    try:
      return ost.geom.Vec3(p[0],p[1],p[2])
    except:
      raise TypeError("expected either a sequence or a geom.Vec3 object")
  

def _primlist_add_point(self,pos,color=None):
  pos=_to_vec3(pos)
  if not color:
    color=WHITE
  self._add_point(pos,color)
  
def _primlist_add_line(self,pos1,pos2,color1=None,color2=None,color=None):
  pos1=_to_vec3(pos1)
  pos2=_to_vec3(pos2)
  if not color:
    color=WHITE
  if not color1:
    color1=color
  if not color2:
    color2=color1
  self._add_line(pos1,pos2,color1,color2)

def _primlist_add_sphere(self,cen,radius=1.0,color=None):
  pos=_to_vec3(cen)
  if not color:
    color=WHITE
  self._add_sphere(pos,radius,color)
  
def _primlist_add_cyl(self,pos1,pos2,radius1=None,radius2=None,radius=None,color1=None,color2=None,color=None,):
  pos1=_to_vec3(pos1)
  pos2=_to_vec3(pos2)
  if radius is None:
    radius=1.0
  if radius1 is None:
    radius1=radius
  if radius2 is None:
    radius2=radius1
  if not color:
    color=WHITE
  if not color1:
    color1=color
  if not color2:
    color2=color1
  self._add_cyl(pos1,pos2,radius1,radius2,color1,color2)

def _primlist_add_text(self,text,pos,color=None,point_size=None):
  pos=_to_vec3(pos)
  if not color:
    color=WHITE
  if not point_size:
    point_size=1.0
  self._add_text(text,pos,color,point_size)

PrimList.AddPoint=_primlist_add_point
PrimList.AddLine=_primlist_add_line
PrimList.AddSphere=_primlist_add_sphere
PrimList.AddCyl=_primlist_add_cyl
PrimList.AddText=_primlist_add_text

# entity reset

def _entity_reset(self,*args,**kwargs):
  import ost.mol as mol
  eh=None
  ev=None
  qr=None
  qf=None
  for a in args:
    if isinstance(a,mol.Query):
      if qr:
        raise TypeError("Reset: more than one query string given")
      qr=a
    elif isinstance(a,mol.EntityHandle):
      if eh:
        raise TypeError("Reset: more than one entity handle given")
      eh=a
    elif isinstance(a,mol.EntityView):
      if ev:
        raise TypeError("Reset: more than one entity view given")
      ev=a
    elif isinstance(a,str):
      if qr:
        raise TypeError("Reset: more than one query string given")
      qr=mol.Query(a)
    elif isinstance(a,int):
      if qf:
        raise TypeError("Reset: more than one QueryFlags given")
      qf=a
    else:
      raise TypeError("Reset: unknown option of type '%s' given"%type(a))

  for key,val in kwargs.items():
    if key=="entity":
      if not isinstance(val,mol.EntityHandle):
        raise TypeError("Reset: expected mol.EntityHandle for 'entity' option")
      if eh:
        raise TypeError("Reset: more than one entity handle given")
      eh=val
    elif key=="view":
      if not isinstance(val,mol.EntityView):
        raise TypeError("Reset: expected mol.EntityView for 'view' option")
      if ev:
        raise TypeError("Reset: more than one entity view given")
      ev=val
    elif key=="query":
      if isinstance(val,mol.Query):
        pass
      elif isinstance(val,str):
        val=mol.Query(val)
      else:
        raise TypeError("Reset: expected mol.Query or string for 'query' option")
      if qr:
        raise TypeError("Reset: more than one query string given")
      qr=val
    elif key=="query_flags":
      if not isinstance(val,int):
        raise TypeError("Reset: expected integer for 'query_flags' option")
      if qf:
        raise TypeError("Reset: more than one query flags given")
      qf=val
    else:
      raise TypeError("Reset: unknown key '%s'"%key)
        
  if eh and ev:
    raise TypeError("Reset: entity and view are mutually exclusive options")

  if ev:
    self._reset4(ev)
  else:
    if not eh:
      eh = self.query_view.entity
    if not qr:
      qr = self.query_view.query
    if not qf:
      qf = self.query_view.GetFlags()
    self._reset3(eh,qr,qf)

Entity.Reset=_entity_reset

def _scene_export(self,*args,**kwargs):
  """
  scene.Export(Exporter)
  scene.Export("file.png")
  scene.Export("file.png",(width,height),samples=0,transparency=False)
  deprecated:
  scene.Export("file.png",width,height,samples=0,transparency=False)
  scene.Export("file.png",width,height,transparency) 
  """
  scene=Scene()
  tp=False
  sa=0
  if "tp" in kwargs:
    tp=int(kwargs["tp"])
  if "transparency" in kwargs:
    tp=int(kwargs["transparency"])
  if "samples" in kwargs:
    sa=int(kwargs["samples"])

  if len(args)==1:
    if isinstance(args[0],Exporter):
      scene._export_via_exporter(args[0])
      return
    elif type(args[0])==type(""):
      scene._export_screen(args[0],tp)
      return
  elif len(args)==2:
    if type(args[0]==type("")):
      # assume second argument is a dimension
      width=int(args[1][0])
      height=int(args[1][1])
      scene._export_buffer(args[0],width,height,sa,tp)
      return
  elif len(args)==3:
    if type(args[0]==type("")):
      width=int(args[1])
      height=int(args[2])
      scene._export_buffer(args[0],width,height,sa,tp)
      return
  elif len(args)==4:
    if type(args[0]==type("")):
      width=int(args[1])
      height=int(args[2])
      tp=int(args[3])
      scene._export_buffer(args[0],width,height,sa,tp)
      return
  # getting here indicates an error
  raise RuntimeError("""invalid arguments to scene.Export; expected one of
  Export(gfx.Exporter)
  Export('file.png')
  Export('file.png',(width,height),samples=0, transparency=False)
  Export('file.png',width,height,samples=0, transparency=False) -> deprecated
  Export('file.png',width,height,transparency) -> deprecated
  """)

SceneSingleton.Export=_scene_export

import __main__ as main_mod
main_mod.scene=Scene()
main_mod.scene.Stereo=Stereo

import ost as ost_mod
ost_mod.scene=Scene()
ost_mod.scene.Stereo=Stereo

def GostExporter(file,scale=1.0,to_origin=True):
  e=GostExporter_(file)
  e.scale=scale
  e.to_origin=to_origin
  return e

def ColladaExporter(file,scale=1.0,to_origin=True):
  e=ColladaExporter_(file)
  e.scale=scale
  e.to_origin=to_origin
  return e

def _go_get_vis(go):
  return go.IsVisible()

def _go_set_vis(go,flag):
  if flag:
    go.Show()
  else:
    go.Hide()

GfxObj.visible=property(_go_get_vis,_go_set_vis)





"""
Interface to LGA

Author: Marco Biasini
"""
import tempfile
import os
from ost import io
import subprocess
from ost import geom
from ost import settings

def _FindLGABinary(lga_bin):
  return settings.Locate('lga', explicit_file_name=lga_bin, 
                         env_name='LGA_BINARY')

def _PrepareInput(pdb1, pdb2, output_pdb):
  """
  Deal with the brain-dead input requirements of LGA.
  """
  mol1=os.path.basename(os.path.splitext(pdb1)[0])
  mol2=os.path.basename(os.path.splitext(pdb2)[0])  
  os.system('echo MOLECULE %s > %s' % (mol1, output_pdb))
  os.system('cat %s >> %s' % (pdb1, output_pdb))
  os.system('echo "\n" >> %s' % (output_pdb))  
  os.system('echo MOLECULE %s >> %s' % (mol2, output_pdb))
  os.system('cat %s >> %s' % (pdb2, output_pdb))  

class GDTResult:
  def __init__(self, rotation, shift, gdt_ts, gdt_ha):
    self.rotation=rotation
    self.shift=shift
    self.gdt_ts=gdt_ts
    self.gdt_ha=gdt_ha
  def GetTransform(self):
    transform=geom.Mat4()
    transform.PasteTranslation(self.shift)
    return transform*geom.Mat4(self.rotation)

def _ParseRotationAndShift(lines):
  t=[l.split() for l in lines]
  rot=geom.Mat3()
  shift=geom.Vec3()
  for i, x in enumerate(t):
    rot[(i, 0)]=+float(x[2])
    rot[(i, 1)]=+float(x[6])
    rot[(i, 2)]=+float(x[10])
    shift[i]=float(x[14])
  return rot, shift

def _ParseGDTSection(section, residue_count):
  cutoffs=[float(e) for e in section[0].split()[2:]]
  num_ca=[int(e) for e in section[1].split()[2:]]
  gdt_ts=[float(e) for e in section[2].split()[2:]]
  scores=dict(list(zip(cutoffs, gdt_ts)))
  numbers=dict(list(zip(cutoffs, num_ca)))
  factor=(1.0/(4*residue_count))*100
  ts_cutoffs=(1.0, 2.0, 4.0, 8.0)
  ha_cutoffs=(0.5, 1.0, 2.0, 4.0)  
  gdt_ts=(sum([numbers[c] for c in ts_cutoffs]))*factor
  gdt_ha=(sum([numbers[c] for c in ha_cutoffs]))*factor  
  return gdt_ts, gdt_ha 

def _ParseLGAOutput(output, residue_count):
  result=GDTResult(geom.Mat3(), geom.Vec3(), 0.0, 0.0)
  found_gdt_section=False
  found_transform_section=False
  for index, line in enumerate(output):
    if line.startswith('GLOBAL_DISTANCE_TEST'): 
      next_lines=output[index+1:index+5]
      result.gdt_ts, result.gdt_ha=_ParseGDTSection(next_lines, residue_count)
      found_gdt_section=True      
    if line.startswith('Unitary ROTATION matrix'):
      next_lines=output[index+1:index+4]
      result.rotation, result.shift=_ParseRotationAndShift(next_lines)
      found_transform_section=True
      break
  assert found_transform_section and found_gdt_section
  return result
  
def GDT(pdb1, pdb2, chain1='', chain2='', reference_length=None, lga_bin=None):
  """
  Calculate GDT value between pdb1 and pdb2. It is assumed that the
  corresponding residues in pdb1 and pdb2 have the same residue numbers.
  """
  lga_bin=_FindLGABinary(lga_bin)  
  temp_d=tempfile.mkdtemp(prefix='lga_gdt_ts_')
  for d in ('MOL2', 'TMP', 'RESULTS'):
    os.mkdir(os.path.join(temp_d, d))

  if not chain1:
    chain1=pdb1.chains[0].name
  if not chain2:
    chain2=pdb2.chains[0].name
  pdb_one_name=os.path.join(temp_d, 'MOL2', 'one.pdb')
  pdb_two_name=os.path.join(temp_d, 'MOL2', 'two.pdb')
  io.SaveEntity(pdb1, pdb_one_name)
  io.SaveEntity(pdb2, pdb_two_name)
  _PrepareInput(pdb_one_name, pdb_two_name, 
                os.path.join(temp_d, 'MOL2', 'input.pdb'))
  output_file=os.path.join(temp_d, 'out.txt')
  ch1, ch2=('', '')
  if len(chain1.strip()): ch1=' -ch1:%s ' % chain1
  if len(chain2.strip()): ch2=' -ch2:%s ' % chain2
    
  params=(temp_d, lga_bin, 'input.pdb', ch1, ch2)  
  command='cd %s; %s %s %s%s-ie -3 -d:4 -sda'

  expanded_cmd=command % params
  lga_proc=subprocess.Popen(expanded_cmd, shell=True, 
                            stdout=subprocess.PIPE)
  stdout, _ = lga_proc.communicate()

  length=reference_length or max(pdb1.residue_count, pdb2.residue_count)
  result=_ParseLGAOutput(stdout.decode().splitlines(), reference_length)
  os.system('rm -r %s' % temp_d)
  return result



"""
Code coverage report generator for Ost
"""

import optparse
import glob
import os, os.path

def Which(program_name):
  """
  emulate unit which program
  """
  for path in os.environ['PATH'].split(os.pathsep):
    path=os.path.expanduser(os.path.expandvars(path))
    full=os.path.join(path, program_name)
    if os.path.exists(full) and os.access(full, os.X_OK):
      return full
  return None

def SubDirsOf(directory):
  directories=[]
  for d in glob.glob(os.path.join(directory, '*')):
      if os.path.isdir(d):
        directories.append(os.path.basename(d))
  return directories
  
def ListModules():
  return ','.join(SubDirsOf('modules'))

def ParseOptions():
  parser=optparse.OptionParser()
  parser.add_option('-m', '--modules', dest='modules',
                    help='modules to test, separated by \',\'')
  parser.add_option('-o', '--output-directory', dest='output_dir',
                    type='string', help='output directory')

  parser.set_defaults(modules=ListModules())
  parser.set_defaults(output_dir='LCOV')
  (options, args)=parser.parse_args()
  options.modules=options.modules.split(',')
  return options

def CheckLCOV():
  if not Which('lcov'):
    print('please install lcov to run this script')
    sys.exit(-1)

CheckLCOV()

opts=ParseOptions()

def Print(message):
  print('~~~~~~')
  print('|  *  |', message)
  print('~~~~~~')  
  
def RemoveFiles(directory, extension):
  glob_pattern='*.%s' % extension
  for f in glob.glob(os.path.join(directory, glob_pattern)):
    os.unlink(f)

def Cleanup(modules):
  for module in modules:
    sub_dirs=SubDirsOf(os.path.join('modules', module))
    for sub_dir in sub_dirs:
      RemoveFiles(os.path.join('modules',module, sub_dir), 'gcda')

def RunLCOV(module):
  Print('Running Tests for %s' % module)  
  sub_dirs=SubDirsOf(os.path.join('modules', module))
  for sub_dir in sub_dirs:
    if sub_dir=='pymod':
      continue
    full_dir=os.path.join('modules', module, sub_dir)
    out='lcov-tmp/%s_%s.info' % (module, sub_dir)
    cmd='lcov --capture --b . --directory "%s" -o "%s"' % (full_dir, out)
    os.system(cmd+' > /dev/null')
  files_to_merge=glob.glob('lcov-tmp/%s_*.info' % module)
  output_name='lcov-tmp/%s_merge.info' % module  

  merge_cmd='lcov %s -o %s' % (' '.join(['-a %s' % f for f in files_to_merge]),
                               output_name)
  os.system(merge_cmd+' > /dev/null')
  
  libs_to_remove=['"*/boost/*"', '"/usr/*"', '"/opt*"']
  remove_cmd='lcov -r %s %s -o %s ' % (output_name, ' '.join(libs_to_remove),
                                       output_name)
  os.system(remove_cmd+' > /dev/null')
  

def MergeAll(modules):
  filtered_modules=[]
  for f in modules:
    if os.access('lcov-tmp/%s_merge.info' % f, os.F_OK):
      filtered_modules.append(f)
  
  f=' '.join(['-a lcov-tmp/%s_merge.info' % f for f in filtered_modules])
  merge_cmd='lcov %s -o %s' % (f, 'lcov-tmp/merge.info')
  os.system(merge_cmd+' > /dev/null')

def GenReport(output_dir):
  os.system('rm -rf LCOV')
  cmd='genhtml -o %s lcov-tmp/merge.info' % (output_dir)
  os.system(cmd+'> /dev/null')

def RunTests(modules):
  try:
    os.mkdir('lcov-tmp')
  except:
    pass
  for module in modules:
    test_dir=os.path.join('modules', module, 'tests')
    if os.path.exists(test_dir):
      test_binary=os.path.join(os.path.join(test_dir, 'tests'))
      if os.path.exists(test_binary) and os.access(test_binary, os.X_OK):
        print('running tests for module', module)        
        Cleanup(modules)
        cmd='cd "%s"; ./tests' % test_dir
        os.system(cmd)
        RunLCOV(module)
  MergeAll(modules)
  
Print('recompiling project with --test-coverage, if neccessary')
os.system('scons --test-coverage unit_tests > /dev/null')  
RunTests(opts.modules)
Print('Generating report in HTML format')
GenReport(opts.output_dir)



import struct

with open("test.gost","rb") as gf:

  header = gf.read(6)
  if str(header[0:4]) != "GOST":
    raise RuntimeError("file format mismatch")
  raw = gf.read(16)

  while raw:
    (type, subtype,size) = struct.unpack("iiL",raw)
    print("found type=%d, subtype=%d and blocksize=%u"%(type,subtype,size))
    if size>0:
      data = gf.read(size)
    raw = gf.read(16)



"""
**This Module requires numpy**

This module contains functions to analyze trajectories, mainly 
similiraty measures baed on RMSDS and pairwise distances.

Author: Niklaus Johner (niklaus.johner@unibas.ch)
"""

import ost.mol.alg
import ost.geom
from ost import LogError
import os

def smooth(vec,n):
#Function to smooth a vector or a list of floats
#for each element it takes the average over itself and the
#n elements on each side, so over (2n+1) elements
  try:
    vec2=vec.copy()
  except:
    vec2=vec[:]
  for i in range(n):
    v=0.0
    count=1.0
    v+=vec[i]
    for j in range(n):
      count+=1
      v+=vec[i+j+1]
    for j in range(i):
      count+=1
      v+=vec[i-(j+1)]
    vec2[i]=v/float(count)
  for i in range(1,n+1):
    v=0.0
    count=1.0
    v+=vec[-i]
    for j in range(n):
      count+=1
      v+=vec[-(i+j+1)]
    for j in range(i-1):
      count+=1
      v+=vec[-i+j+1]
    vec2[-i]=v/float(count)
  for i in range(n,len(vec2)-n):
    v=vec[i]
    for j in range(n):
      v+=vec[i+j+1]
      v+=vec[i-j-1]
    vec2[i]=v/float(2.*n+1.)
  return vec2


"""
From here on the module needs numpy
"""

def RMSD_Matrix_From_Traj(t,sele,first=0,last=-1,align=True,align_sele=None):
  """
  This function calculates a matrix M such that M[i,j] is the
  RMSD (calculated on **sele**) between frames i and j of the trajectory **t**
  aligned on sele.

  :param t: the trajectory
  :param sele: the selection used for alignment and RMSD calculation
  :param first: the first frame of t to be used
  :param last: the last frame of t to be used
  :type t: :class:`~ost.mol.CoordGroupHandle`
  :type sele: :class:`~ost.mol.EntityView`
  :type first: :class:`int`
  :type last: :class:`int`

  :return: Returns a numpy N\ :subscript:`frames`\ xN\ :subscript:`frames` matrix, 
   where N\ :subscript:`frames` is the number of frames.
  """
  if not align_sele:align_sele=sele
  try:
    import numpy as npy
    if last==-1:last=t.GetFrameCount()
    n_frames=last-first
    rmsd_matrix=npy.identity(n_frames)
    for i in range(n_frames):
      if align:
        t=ost.mol.alg.SuperposeFrames(t,align_sele,begin=first,end=last,ref=i)
        eh=t.GetEntity()
      t.CopyFrame(i)
      rmsd_matrix[i,:]=ost.mol.alg.AnalyzeRMSD(t,sele,sele)
      if i==0:
        last=last-first
        first=0
    return rmsd_matrix
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise


def PairwiseDistancesFromTraj(t,sele,first=0,last=-1,seq_sep=1):
  """
  This function calculates the distances between any pair of atoms in **sele**  
  with sequence separation larger than **seq_sep** from a trajectory **t**.
  It return a matrix containing one line for each atom pair and N\ :subscript:`frames` columns, where
  N\ :subscript:`frames` is the number of frames in the trajectory.
  
  :param t: the trajectory
  :param sele: the selection used to determine the atom pairs
  :param first: the first frame of t to be used
  :param last: the last frame of t to be used
  :param seq_sep: The minimal sequence separation between atom pairs
  :type t: :class:`~ost.mol.CoordGroupHandle`
  :type sele: :class:`~ost.mol.EntityView`
  :type first: :class:`int`
  :type last: :class:`int`
  :type seq_sep: :class:`int`

  :return: a numpy N\ :subscript:`pairs`\ xN\ :subscript:`frames` matrix.
  """
  try:
    import numpy as npy
    if last==-1:last=t.GetFrameCount()
    n_frames=last-first
    n_var=0
    for i,a1 in enumerate(sele.atoms):
      for j,a2 in enumerate(sele.atoms):
        if not j-i<seq_sep:n_var+=1
    #n_var=sele.GetAtomCount()
    #n_var=(n_var-1)*(n_var)/2.
    dist_matrix=npy.zeros(n_frames*n_var)
    dist_matrix=dist_matrix.reshape(n_var,n_frames)
    k=0
    for i,a1 in enumerate(sele.atoms):
      for j,a2 in enumerate(sele.atoms):
        if j-i<seq_sep:continue
        dist_matrix[k]=ost.mol.alg.AnalyzeDistanceBetwAtoms(t,a1.GetHandle(),a2.GetHandle())[first:last]
        k+=1
    return dist_matrix
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise
    
def DistanceMatrixFromPairwiseDistances(distances,p=2):
  """
  This function calculates an distance matrix M(N\ :subscript:`frames`\ xN\ :subscript:`frames`\ ) from
  the pairwise distances matrix D(N\ :subscript:`pairs`\ xN\ :subscript:`frames`\ ), where 
  N\ :subscript:`frames` is the number of frames in the trajectory
  and N\ :subscript:`pairs` the number of atom pairs.
  M[i,j] is the distance between frame i and frame j
  calculated as a p-norm of the differences in distances
  from the two frames (distance-RMSD for p=2).

  :param distances: a pairwise distance matrix as obtained from 
   :py:func:`~mol.alg.trajectory_analysis.PairwiseDistancesFromTraj`
  :param p: exponent used for the p-norm.

  :return: a numpy N\ :subscript:`frames`\ xN\ :subscript:`frames` matrix, where N\ :subscript:`frames`
   is the number of frames.
  """
  try:
    import numpy as npy
    n1=distances.shape[0]
    n2=distances.shape[1]
    dist_mat=npy.identity(n2)
    for i in range(n2):
      for j in range(n2):
        if j<=i:continue
        d=(((abs(distances[:,i]-distances[:,j])**p).sum())/float(n1))**(1./p)
        dist_mat[i,j]=d
        dist_mat[j,i]=d
    return dist_mat
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise

def DistRMSDFromTraj(t,sele,ref_sele,radius=7.0,average=False,seq_sep=4,first=0,last=-1):
  """
  This function calculates the distance RMSD from a trajectory.
  The distances selected for the calculation are all the distances
  between pair of atoms from residues that are at least **seq_sep** apart
  in the sequence and that are smaller than **radius** in **ref_sel**.
  The number and order of atoms in **ref_sele** and **sele** should be the same.

  :param t: the trajectory
  :param sele: the selection used to calculate the distance RMSD
  :param ref_sele: the reference selection used to determine the atom pairs and reference distances
  :param radius: the upper limit of distances in ref_sele considered for the calculation
  :param seq_sep: the minimal sequence separation between atom pairs considered for the calculation 
  :param average: use the average distance in the trajectory as reference instead of the distance obtained from ref_sele
  :param first: the first frame of t to be used
  :param last: the last frame of t to be used
  
  :type t: :class:`~ost.mol.CoordGroupHandle`
  :type sele: :class:`~ost.mol.EntityView`
  :type ref_sele: :class:`~ost.mol.EntityView`
  :type radius: :class:`float`
  :type average: :class:`bool`
  :type first: :class:`int`
  :type last: :class:`int`
  :type seq_sep: :class:`int`

  :return: a numpy vecor dist_rmsd(N\ :subscript:`frames`).  
  """
  if not sele.GetAtomCount()==ref_sele.GetAtomCount():
    print('Not same number of atoms in the two views')
    return
  try:
    import numpy as npy
    if last==-1:last=t.GetFrameCount()
    n_frames=last-first
    dist_rmsd=npy.zeros(n_frames)
    pair_count=0.0
    for i,a1 in enumerate(ref_sele.atoms):
      for j,a2 in enumerate(ref_sele.atoms):
        if j<=i:continue
        r1=a1.GetResidue()
        c1=r1.GetChain()
        r2=a2.GetResidue()
        c2=r2.GetChain()      
        if c1==c2 and abs(r2.GetNumber().num-r1.GetNumber().num)<seq_sep:continue
        d=ost.geom.Distance(a1.pos,a2.pos)
        if d<radius:
          a3=sele.atoms[i]
          a4=sele.atoms[j]
          d_traj=ost.mol.alg.AnalyzeDistanceBetwAtoms(t,a3.GetHandle(),a4.GetHandle())[first:last]
          if average:d=npy.mean(d_traj)
          for k,el in enumerate(d_traj):
            dist_rmsd[k]+=(el-d)**2.0
          pair_count+=1.0
    return (dist_rmsd/float(pair_count))**0.5
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise
    
def AverageDistanceMatrixFromTraj(t,sele,first=0,last=-1):
  """
  This function calcultes the distance between each pair of atoms
  in **sele**, averaged over the trajectory **t**.

  :param t: the trajectory
  :param sele: the selection used to determine the atom pairs
  :param first: the first frame of t to be used
  :param last: the last frame of t to be used
  :type t: :class:`~ost.mol.CoordGroupHandle`
  :type sele: :class:`~ost.mol.EntityView`
  :type first: :class:`int`
  :type last: :class:`int`

  :return: a numpy N\ :subscript:`pairs`\ xN\ :subscript:`pairs` matrix, where N\ :subscript:`pairs`
   is the number of atom pairs in **sele**.
  """
  try:
    import numpy as npy
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise
  n_atoms=sele.GetAtomCount()
  M=npy.zeros([n_atoms,n_atoms])
  for i,a1 in enumerate(sele.atoms):
    for j,a2 in enumerate(sele.atoms):
      if j>i:continue
      d=ost.mol.alg.AnalyzeDistanceBetwAtoms(t,a1.GetHandle(),a2.GetHandle())[first:last]
      M[i,j]=npy.mean(d)
      M[j,i]=npy.mean(d)
  return M

def AnalyzeDistanceFluctuationMatrix(t,sele,first=0,last=-1):
  try:
    import numpy as npy
  except ImportError:
    LogError("Function needs numpy, but I could not import it.")
    raise
  n_atoms=sele.GetAtomCount()
  M=npy.zeros([n_atoms,n_atoms])
  for i,a1 in enumerate(sele.atoms):
    for j,a2 in enumerate(sele.atoms):
      if i>j:continue
      d=ost.mol.alg.AnalyzeDistanceBetwAtoms(t,a1.GetHandle(),a2.GetHandle())[first:last]
      M[j,i]=npy.std(d)
      M[i,j]=npy.std(d)
  return M
  
def IterativeSuperposition(t,sele,threshold=1.0,initial_sele=None,iterations=5,ref_frame=0):
  if initial_sele:current_sele=initial_sele
  else: current_sele=sele
  for i in range(iterations):
    t=ost.mol.alg.SuperposeFrames(t,current_sele,ref=ref_frame)
    al=[a for a in sele.atoms if ost.mol.alg.AnalyzeRMSF(t,ost.mol.CreateViewFromAtoms([a]))<threshold]
    if len(al)==0:return
    current_sele=ost.mol.CreateViewFromAtoms(al)
  return current_sele
    


from ost.mol.mm import *
from PyQt5 import QtCore

"""
MM - Demo

!!!Requires OpenStructure to be compiled with MM-Support!!!

This demo should show the procedure of building a custom forcefield.
The script can directly be started from within DNG.
"""

class Anim(QtCore.QTimer):
    def __init__(self,sim,go):
        QtCore.QTimer.__init__(self)
        self.ent=sim.GetEntity()
        self.sim=sim
        self.go = go
        self.ed = ent.EditXCS()
        self.timeout.connect(self.OnTimer)
        
    def OnTimer(self):
        self.sim.Steps(1)
        self.sim.UpdatePositions()
        self.go.UpdatePositions()


#Let's create an ethanol without its hydrogens
ent = ost.mol.CreateEntity()
ed = ent.EditXCS()
chain = ed.InsertChain('A')
res = ed.AppendResidue(chain,"ETH")
c1 = ed.InsertAtom(res,"C1",geom.Vec3(-0.246,0.0,1.285),"C")
c2 = ed.InsertAtom(res,"C2",geom.Vec3(0.566,0.0,-0.011),"C")
o = ed.InsertAtom(res,"O",geom.Vec3(-0.322,0.0,-1.130),"O")
ed.Connect(c1,c2)
ed.Connect(c2,o)


ff = Forcefield()

#generate hydrogen constructor and add it to the forcefield
h_constructor = GromacsHydrogenConstructor()
h_constructor.AddHydrogenRule(3,4,["H11","H12","H13"],["C1","C2","O"])
h_constructor.AddHydrogenRule(2,6,["H21","H22"],["C2","O","C1"])
h_constructor.AddHydrogenRule(1,2,["OH"],["O","C2","C1"])
ff.AddHydrogenConstructor("ETH",h_constructor)

#generate buildingblock and add it to the forcefield
building_block = BuildingBlock()
building_block.AddAtom("C1","C",-0.18)
building_block.AddAtom("C2","C",0.145)
building_block.AddAtom("O","O",-0.683)
building_block.AddAtom("H11","H",0.06)
building_block.AddAtom("H12","H",0.06)
building_block.AddAtom("H13","H",0.06)
building_block.AddAtom("H21","H",0.06)
building_block.AddAtom("H22","H",0.06)
building_block.AddAtom("OH","OH",0.418)

bonds = list()
for i in range(8):
  bonds.append(Interaction(FuncType.HarmonicBond))

bonds[0].SetNames(["H11","C1"])
bonds[1].SetNames(["H12","C1"])
bonds[2].SetNames(["H13","C1"])
bonds[3].SetNames(["C1","C2"])
bonds[4].SetNames(["C2","H21"])
bonds[5].SetNames(["C2","H22"])
bonds[6].SetNames(["O","C2"])
bonds[7].SetNames(["O","OH"])

for b in bonds:
  building_block.AddBond(b)

ff.AddBuildingBlock("ETH",building_block)

#add masses to the forcefield
ff.AddMass("C",12.011)
ff.AddMass("O",15.999)
ff.AddMass("H",1.008)
ff.AddMass("OH",1.008)

#generate bonds and add them to the forcefield
h_c_bond = Interaction(FuncType.HarmonicBond)
c_c_bond = Interaction(FuncType.HarmonicBond)
c_o_bond = Interaction(FuncType.HarmonicBond)
o_h_bond = Interaction(FuncType.HarmonicBond)

h_c_bond.SetTypes(["H","C"])
c_c_bond.SetTypes(["C","C"])
c_o_bond.SetTypes(["C","O"])
o_h_bond.SetTypes(["O","OH"])

h_c_bond.SetParam([0.1111,269449.6])
c_c_bond.SetParam([0.153,186188.0])
c_o_bond.SetParam([0.142,358150.4])
o_h_bond.SetParam([0.096,456056.0])

ff.AddBond(h_c_bond)
ff.AddBond(c_c_bond)
ff.AddBond(c_o_bond)
ff.AddBond(o_h_bond)

#generate angles and add them to the forcefield
h_c_h_angle = Interaction(FuncType.UreyBradleyAngle)
h_c_c_angle = Interaction(FuncType.UreyBradleyAngle)
h_c_o_angle = Interaction(FuncType.UreyBradleyAngle)
c_c_o_angle = Interaction(FuncType.UreyBradleyAngle)
c_o_h_angle = Interaction(FuncType.UreyBradleyAngle)

h_c_h_angle.SetTypes(["H","C","H"])
h_c_c_angle.SetTypes(["H","C","C"])
h_c_o_angle.SetTypes(["H","C","O"])
c_c_o_angle.SetTypes(["C","C","O"])
c_o_h_angle.SetTypes(["C","O","OH"])

h_c_h_angle.SetParam([1.892,297.064,0.1802,4518.72])
h_c_c_angle.SetParam([1.922,289.5328,0.2179,18853.104])
h_c_o_angle.SetParam([1.900,384.0912,0.0,0.0])
c_c_o_angle.SetParam([1.920,633.4576,0.0,0.0])
c_o_h_angle.SetParam([1.850,481.16,0.0,0.0])

ff.AddAngle(h_c_h_angle)
ff.AddAngle(h_c_c_angle)
ff.AddAngle(h_c_o_angle)
ff.AddAngle(c_c_o_angle)
ff.AddAngle(c_o_h_angle)

#generate dihedrals and add them to the forcefield
x_c_c_x_dihedral = Interaction(FuncType.PeriodicDihedral)
c_c_o_h_dihedral_one = Interaction(FuncType.PeriodicDihedral)
c_c_o_h_dihedral_two = Interaction(FuncType.PeriodicDihedral)
c_c_o_h_dihedral_three = Interaction(FuncType.PeriodicDihedral)
x_c_o_x_dihedral = Interaction(FuncType.PeriodicDihedral)

x_c_c_x_dihedral.SetTypes(["X","C","C","X"])
c_c_o_h_dihedral_one.SetTypes(["C","C","O","OH"])
c_c_o_h_dihedral_two.SetTypes(["C","C","O","OH"])
c_c_o_h_dihedral_three.SetTypes(["C","C","O","OH"])
x_c_o_x_dihedral.SetTypes(["X","C","O","X"])

x_c_c_x_dihedral.SetParam([3,0.0,0.66944])
c_c_o_h_dihedral_one.SetParam([1,0.0,5.4392])
c_c_o_h_dihedral_two.SetParam([2,0.0,1.2552])
c_c_o_h_dihedral_three.SetParam([3,0.0,1.75728])
x_c_o_x_dihedral.SetParam([3,0.0,0.58576])

ff.AddDihedral(x_c_c_x_dihedral)
ff.AddDihedral(c_c_o_h_dihedral_one)
ff.AddDihedral(c_c_o_h_dihedral_two)
ff.AddDihedral(c_c_o_h_dihedral_three)
ff.AddDihedral(x_c_o_x_dihedral)

#add lj parameters
c_lj = Interaction(FuncType.LJ)
o_lj = Interaction(FuncType.LJ)
h_lj = Interaction(FuncType.LJ)
oh_lj = Interaction(FuncType.LJ)

c_lj.SetTypes(["C"])
o_lj.SetTypes(["O"])
h_lj.SetTypes(["H"])
oh_lj.SetTypes(["OH"])

c_lj.SetParam([0.37,0.28])
o_lj.SetParam([0.32,0.64])
h_lj.SetParam([0.24,0.09])
oh_lj.SetParam([0.04,0.2])

ff.AddLJ(c_lj)
ff.AddLJ(o_lj)
ff.AddLJ(h_lj)
ff.AddLJ(oh_lj)

#we do not explicitely set the 1,4 interactions,
#we rather decrease it by a certain factor
ff.SetFudgeLJ(0.5)
ff.SetFudgeQQ(0.5)

#construct settings
settings = Settings()
settings.init_temperature =  310
settings.forcefield = ff

#construct integrator with tiny timesteps
settings.integrator = VerletIntegrator(0.0001)


#create the simulation
sim = Simulation(ent,settings)

io.SavePDB(sim.GetEntity(),'ethanol.pdb')


go = gfx.Entity("ethanol",sim.GetEntity())
go.SetRenderMode(gfx.CUSTOM)
scene.Add(go)

anim = Anim(sim,go)
anim.start(2)





import random
prims1 = gfx.PrimList("prims one")
prims2 = gfx.PrimList("prims two")
for i in range(1000):
    prims1.AddLine(10.0*geom.Vec3(random.random()-1,
                                 random.random()-1,
                                 random.random()-0.5),
                  10.0*geom.Vec3(random.random()-0.5,
                                 random.random()-0.5,
                                 random.random()-0.5),
                  gfx.Color(random.random(),
                            random.random(),
                            0.5)
                  );
    prims2.AddLine(10.0*geom.Vec3(random.random(),
                                 random.random(),
                                 random.random()-0.5),
                  10.0*geom.Vec3(random.random()-0.5,
                                 random.random()-0.5,
                                 random.random()-0.5),
                  gfx.Color(random.random(),
                            0.5,
                            random.random())
                  );
scene.Add(prims1)
scene.Add(prims2)
scene.SetCenter(geom.Vec3(0,0,0))



                            
                  
                            



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import info
from ost import gfx

from ost.gfx import ColorOp

#Rendering Preset
class Preset:
  NAME_ATTRIBUTE_NAME = "Name"
  OP_GROUP_NAME = "Op"
  CLASS_NAME_ATTRIBUTE_NAME = "ClassName"
  INDEX_ATTRIBUTE_NAME = "Index"
  
  MODULE_NAMES = ["ost.gfx","ost.gui.scene.visibility_op","ost.gui.scene.render_op"]
  
  def __init__(self, name, parent=None):    
    self.name_ = name
    self.ops_ = list()
    
  def SetName(self, name):
    self.name_ = name
    
  def GetName(self):
    return self.name_
    
  def InsertOp(self, index, op):
    self.ops_.insert(index, op)
    
  def RemoveOp(self, op):
    self.ops_.remove(op)
  
  def RemoveOpAt(self, index):
    del(self.ops_[index])
    
  def GetOp(self, index):
    return self.ops_[index]
  
  def GetOpCount(self):
    return len(self.ops_)
  
  def SetOp(self, index, op):
    self.ops_[index] = op
  
  def AddOp(self, op):
    self.ops_.append(op)
  
  def GetOps(self):
    return self.ops_
  
  def ApplyOn(self, entity):
    if (entity is not None) and isinstance(entity, gfx.Entity):
      for op in self.ops_:
        if isinstance(op,ColorOp):
          entity.Apply(op)
        else:
          op.ApplyOn(entity)
  
  def ToInfo(self,group):
    group.SetAttribute(Preset.NAME_ATTRIBUTE_NAME, self.name_)
    for i in range(0,len(self.ops_)):
      op_group = group.CreateGroup(Preset.OP_GROUP_NAME)
      op_group.SetAttribute(Preset.INDEX_ATTRIBUTE_NAME, str(i))
      op_group.SetAttribute(Preset.CLASS_NAME_ATTRIBUTE_NAME, "%s"%(self.ops_[i].__class__.__name__))
      self.ops_[i].ToInfo(op_group)
    
  @staticmethod
  def FromInfo(group):
    preset = None
    if group.HasAttribute(Preset.NAME_ATTRIBUTE_NAME):
      name = group.GetAttribute(Preset.NAME_ATTRIBUTE_NAME)
      preset = Preset(name)
      group_list = group.GetGroups(Preset.OP_GROUP_NAME)
      
      class_order_dict = dict()
      for op_group in group_list:
        if(op_group.HasAttribute(Preset.CLASS_NAME_ATTRIBUTE_NAME) and op_group.HasAttribute(Preset.INDEX_ATTRIBUTE_NAME)):
          class_name = op_group.GetAttribute(Preset.CLASS_NAME_ATTRIBUTE_NAME)
          index = int(op_group.GetAttribute(Preset.INDEX_ATTRIBUTE_NAME))
          op_class = None
          for module in Preset.MODULE_NAMES:
            try:
              op_class = Preset.__get_op_class("%s.%s"%(module,class_name))
              break
            except AttributeError:
              pass
          if op_class is not None:
            op = op_class.FromInfo(op_group)
            class_order_dict[index]=op
    for i in range(0, len(class_order_dict)):
      if(i in class_order_dict):
        preset.AddOp(class_order_dict[i])
    return preset
    
  @staticmethod
  def __get_op_class( cls ):
    parts = cls.split('.')
    module = ".".join(parts[:-1])
    m = __import__( module )
    for comp in parts[1:]:
      m = getattr(m, comp)
    return m



import unittest
from ost import *
from ost import settings
from ost import seq


def fixture():
  e=mol.CreateEntity()
  ede=e.EditXCS()
  chain=ede.InsertChain('A')
  for res in 'ABCDEFGH':
    r=ede.AppendResidue(chain, res)
    r.SetOneLetterCode(res)
    ede.InsertAtom(r, "XXX", geom.Vec3())
    
  return e
  
class TestSeq(unittest.TestCase):
  
  def setUp(self):
    self.ent=fixture()
    
  def testSeqSlice(self):
    seq_string='abcdefg'
    s=seq.CreateSequence('A', seq_string)
    self.assertEqual(s[1:5], seq_string[1:5])
    self.assertEqual(s[:-1], seq_string[:-1])
    self.assertEqual(s[-3:-2], seq_string[-3:-2])
    self.assertEqual(s[-3:], seq_string[-3:])
    self.assertEqual(s[3:4], seq_string[3:4])
    
  def testSeqListSlice(self):
    a=seq.CreateSequence('A','aaaa')
    b=seq.CreateSequence('B','bbbb')
    c=seq.CreateSequence('C','cccc')
    d=seq.CreateSequence('D','dddd')
    sl=seq.CreateSequenceList(a, b, c, d)
    sliced=sl[1:]
    self.assertEqual(len(sliced), 3)
    self.assertEqual(str(sliced[0]), str(b))
    self.assertEqual(str(sliced[1]), str(c))
    self.assertEqual(str(sliced[2]), str(d))
    sliced=sl[:-1]
    self.assertEqual(len(sliced), 3)
    self.assertEqual(str(sliced[0]), str(a))
    self.assertEqual(str(sliced[1]), str(b))
    self.assertEqual(str(sliced[2]), str(c))
    sliced=sl[-1:]
    self.assertEqual(len(sliced), 1)
    self.assertEqual(str(sliced[0]), str(d))
    
  def testAlnSlice(self):
    a=seq.CreateSequence('A','abcd')
    b=seq.CreateSequence('B','efgh')
    c=seq.CreateSequence('C','ijkl')
    d=seq.CreateSequence('D','mnop')
    aln=seq.CreateAlignment(a, b, c, d)
    sliced=aln[1:]
    self.assertEqual(len(sliced), 3)
    self.assertEqual(str(sliced[0]), 'bfjn')
    self.assertEqual(str(sliced[1]), 'cgko')
    self.assertEqual(str(sliced[2]), 'dhlp')
    sliced=aln[:-1]
    self.assertEqual(len(sliced), 3)
    self.assertEqual(str(sliced[0]), 'aeim')
    self.assertEqual(str(sliced[1]), 'bfjn')
    self.assertEqual(str(sliced[2]), 'cgko')
    sliced=aln[-1:]
    self.assertEqual(len(sliced), 1)
    self.assertEqual(str(sliced[0]), 'dhlp')

  def testViewsFromSequences_01(self):
    seq_a=seq.CreateSequence("A", "ABCD-FGH")
    seq_a.AttachView(self.ent.Select('rname=A,B,C,D,F,G,H'))
    seq_b=seq.CreateSequence("B", "ABCDEFGH")
    seq_b.AttachView(self.ent.Select(''))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABCDFGH')
    self.assertEqual(string_b, 'ABCDFGH')
  def testSeqIterBZDNG148(self):
    s=seq.CreateSequence('A', 'abcdef')
    for x in s: 
      pass
  def testViewsFromSequences_02(self):
    seq_a=seq.CreateSequence("A", "ABCD-FGH")
    seq_a.AttachView(self.ent.Select('rname=A,B,C,D,F,G,H'))
    seq_b=seq.CreateSequence("B", "ABCD-FGH")
    seq_b.AttachView(self.ent.Select('rname=A,B,C,D,F,G,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABCDFGH')
    self.assertEqual(string_b, 'ABCDFGH')
    
  def testViewsFromSequences_03(self):
    seq_a=seq.CreateSequence("A", "ABCD--GH")
    seq_a.AttachView(self.ent.Select('rname=A,B,C,D,G,H'))
    seq_b=seq.CreateSequence("B", "ABCD-FGH")
    seq_b.AttachView(self.ent.Select('rname=A,B,C,D,F,G,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABCDGH')
    self.assertEqual(string_b, 'ABCDGH')
 
  def testViewsFromSequences_04(self):
    seq_a=seq.CreateSequence("A", "ABCD-FGH")
    seq_a.AttachView(self.ent.Select('rname=A,B,C,D,F,G,H'))
    seq_b=seq.CreateSequence("B", "ABCD--GH")
    seq_b.AttachView(self.ent.Select('rname=A,B,C,D,G,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABCDGH')
    self.assertEqual(string_b, 'ABCDGH')
    
  def testViewsFromSequences_05(self):
    seq_a=seq.CreateSequence("A", "ABCD-F--")
    seq_a.AttachView(self.ent.Select('rname=A,B,C,D,F'))
    seq_b=seq.CreateSequence("B", "ABCDEFGH")
    seq_b.AttachView(self.ent.Select('rname=A,B,C,D,E,F,G,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABCDF')
    self.assertEqual(string_b, 'ABCDF')
    
  def testViewsFromSequences_06(self):
    seq_a=seq.CreateSequence("A", "--CD-FGH")
    seq_a.AttachView(self.ent.Select('rname=C,D,F,G,H'))
    seq_b=seq.CreateSequence("B", "ABCDEFGH")
    seq_b.AttachView(self.ent.Select('rname=A,B,C,D,E,F,G,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'CDFGH')
    self.assertEqual(string_b, 'CDFGH')


  def testViewsFromSequences_07(self):
    seq_a=seq.CreateSequence("A", "AB-D-FGH")
    seq_a.AttachView(self.ent.Select('rname=A,B,D,F,G,H'))
    seq_b=seq.CreateSequence("B", "AB-DEF-H")
    seq_b.AttachView(self.ent.Select('rname=A,B,D,E,F,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'ABDFH')
    self.assertEqual(string_b, 'ABDFH')


  def testViewsFromSequences_08(self):
    seq_a=seq.CreateSequence("A", "A-C-E-G")
    seq_a.AttachView(self.ent.Select('rname=A,C,E,G'))
    seq_b=seq.CreateSequence("B", "-B-D-H-")
    seq_b.AttachView(self.ent.Select('rname=B,D,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, '')
    self.assertEqual(string_b, '')

  def testViewsFromSequences_09(self):
    seq_a=seq.CreateSequence("A", "B-D-FGH")
    seq_a.AttachView(self.ent.Select('rname=A,B,D,F,G,H'))
    seq_a.offset=1
    seq_b=seq.CreateSequence("B", "B-DEF-H")
    seq_b.offset=1
    seq_b.AttachView(self.ent.Select('rname=A,B,D,E,F,H'))
    a, b=seq.ViewsFromSequences(seq_a, seq_b)
    string_a=''.join([r.one_letter_code for r in a.residues])
    string_b=''.join([r.one_letter_code for r in b.residues])
    self.assertEqual(string_a, 'BDFH')
    self.assertEqual(string_b, 'BDFH')

  def testNormalise(self):
    seq_a=seq.CreateSequence("A", "B-D-FGH")
    self.assertEqual("B-D-FGH", seq_a.GetString())
    seq_a.Normalise()
    self.assertEqual("BDFGH", seq_a.GetString())
    seq_a=seq.CreateSequence("A", "b.d-fgh")
    self.assertEqual("b.d-fgh", seq_a.GetString())
    seq_a.Normalise()
    self.assertEqual("BDFGH", seq_a.GetString())

if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
"""
Wrappers for the iAlign utility.

References:

Mu Gao and Jeffrey Skolnick, 2010. iAlign: a method for the structural comparison
of protein-protein interfaces. Bioinformatics, 26(18):2259-65.
Mu Gao and Jeffrey Skolnick, 2010. Structural space of protein-protein interfaces
is degenerate, close to complete, and highly connected. PNAS 107(52):22517-22.

Authors: Pascal Benkert, Marco Biasini, Martino Bertoni
"""

import subprocess, os, tempfile, platform
from ost import settings, io, geom, seq

def _SetupFiles(models):
  # create temporary directory
  tmp_dir_name=tempfile.mkdtemp()
  dia = 'PDB'
  for index, model in enumerate(models):
    for chain in model.chains:
      if len(chain.name) > 1:
        dia = 'CHARMM'
        break;
      for res in chain.residues:
        if len(res.name) > 3:
          dia = 'CHARMM'
          break;
    io.SavePDB(model, os.path.join(tmp_dir_name, 'model%02d.pdb' % (index+1)), dialect=dia)
  return tmp_dir_name

def _CleanupFiles(dir_name):
  import shutil
  shutil.rmtree(dir_name)

class iAlignResult:
  """
  Holds the result of running iAlign
  
  .. attribute:: rmsd
    
    The RMSD of the common Calpha atoms of both structures
  
  .. attribute:: transform
  
    The transform that superposes the model onto the reference structure.
    
    :type: :class:`~ost.geom.Mat4`
  
  .. attribute:: alignment
  
    The alignment of the structures, that is the pairing of Calphas of both 
    structures. Since the programs only read ATOM records, residues consisting 
    of HETATMs (MSE) are not included in the alignment.
    
    :type: :class:`~ost.seq.AlignmentHandle`

  .. attribute:: IS_score

    The IS-score of the structural superposition

  .. attribute:: aligned_residues

    The total number of aligned residues

  .. attribute:: aligned_contacts

    The total number of aligned contacts

  """
  def __init__(self, rmsd, transform, alignment, is_score, 
               aligned_residues, aligned_contacts):
    
    self.rmsd=rmsd
    self.transform=transform
    self.alignment=alignment
    self.is_score=is_score    
    self.aligned_residues=aligned_residues
    self.aligned_contacts=aligned_contacts

def _ParseiAlign(lines):
  info_line=lines[18].split(',')
  is_score=float(info_line[0].split('=')[1].strip())
  aln_residues=int(lines[19].split('=')[1].strip())
  aln_contacts=int(lines[20].split('=')[1].strip())
  info_line=lines[21].split(',')
  rmsd=float(info_line[0].split('=')[1].strip())

  tf1=[float(i.strip()) for i in lines[25][1:].split()]
  tf2=[float(i.strip()) for i in lines[26][1:].split()]
  tf3=[float(i.strip()) for i in lines[27][1:].split()]
  rot=geom.Mat3(tf1[2], tf1[3], tf1[4], tf2[2], tf2[3],
                tf2[4], tf3[2], tf3[3], tf3[4])
  tf=geom.Mat4(rot)
  tf.PasteTranslation(geom.Vec3(tf1[1], tf2[1], tf3[1]))
  seq1 = seq.CreateSequence("1",lines[32].strip())
  seq2 = seq.CreateSequence("2",lines[34].strip())
  alignment = seq.CreateAlignment()
  alignment.AddSequence(seq2)
  alignment.AddSequence(seq1)
  return iAlignResult(rmsd, tf, alignment, is_score, aln_residues, aln_contacts)

def _RuniAlign(ialign, tmp_dir, options={}):
  opts = {'a' : 1,  # concise output
          'w' : tmp_dir
         } 
  opts.update(options)
  cmd_opts = []
  for k, v in opts.items():
    if type(v) == type(True):
      if v == True:
        cmd_opts.append('-%s' % str(k))
    else:
      cmd_opts.append('-%s %s' % (str(k), str(v)))
  cmd_opts = ' '.join(cmd_opts)
  model1_filename=os.path.join(tmp_dir, 'model01.pdb')
  model2_filename=os.path.join(tmp_dir, 'model02.pdb')
  if platform.system() == "Windows":
    ialign_path=settings.Locate('ialign.pl', explicit_file_name=ialign)
    command="\"%s\" %s %s %s" % (os.path.normpath(ialign_path), model1_filename, model2_filename, cmd_opts)
  else:
    ialign_path=settings.Locate('ialign.pl', explicit_file_name=ialign)  
    command="\"%s\" \"%s\" \"%s\" %s" % (ialign_path, model1_filename, model2_filename, cmd_opts)

  ps=subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
  stdout,_=ps.communicate()
  lines=stdout.decode().splitlines()

  if (len(lines))<22:
    _CleanupFiles(tmp_dir)
    #for l in lines:
    #  print l
    raise RuntimeError("iAlign superposition failed")
  return _ParseiAlign(lines)


def iAlign(model1, model2, ialign=None):
  """
  Compare protein-protein interfaces of the structures of two pairs of
  protein complexes and suporpose them.
  

  :param model1: The model structure. If the superposition is successful, will 
                 be superposed onto the reference structure
  :type model1: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param model2: The reference structure
  :type model2: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param ialign: If not None, the path to the ialign executable.
  :returns: The result of the tmscore superposition
  :rtype: :class:`iAlignResult`
  
  :raises: :class:`~ost.settings.FileNotFound` if tmalign could not be located.
  :raises: :class:`RuntimeError` if the superposition failed
  """
  tmp_dir_name=_SetupFiles((model1, model2))
  result=_RuniAlign(ialign, tmp_dir_name)
  model1.handle.EditXCS().ApplyTransform(result.transform)
  _CleanupFiles(tmp_dir_name)
  return result



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import mol
from PyQt5 import QtCore, QtGui, QtWidgets
from .color_select_widget import ColorSelectWidget
from .gradient_preset_widget import GradientPresetWidget

#Gradient Editor
class GradientEditor(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    self.parent_ = parent
    
    #Create Ui elements
    gradient_label = QtWidgets.QLabel("Gradient Editor")
    font = gradient_label.font()
    font.setBold(True)
    
    self.prop_combo_box_ = QtWidgets.QComboBox()
    self.gradient_preview_ = GradientPreview()
    self.gradient_edit_ = GradientEdit(self.gradient_preview_,self)
    self.gradient_preset_ = GradientPresetWidget(self.gradient_edit_)
    
    self.prop_combo_box_.addItem("atom B-factor")
    self.prop_combo_box_.addItem("average residue B-factor")
    self.prop_combo_box_.addItem("X-Coordinate")
    self.prop_combo_box_.addItem("Y-Coordinate")
    self.prop_combo_box_.addItem("Z-Coordinate")
    self.prop_combo_box_.addItem("Residue Number")
    self.prop_combo_box_.addItem("Atom Charge")
    
    self.props=["abfac","rbfac","x","y","z","rnum","acharge"]
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(gradient_label, 0, 0, 1, 1)
    grid.addWidget(self.prop_combo_box_, 1, 0, 1, 1)
    grid.addWidget(self.gradient_preview_, 2, 0, 1, 1)
    grid.addWidget(self.gradient_edit_, 3, 0, 1, 1)
    grid.setRowStretch(4, 1)
    grid.addWidget(self.gradient_preset_, 5, 0, 1, 1)
    self.setLayout(grid)
    
    self.prop_combo_box_.currentIndexChanged.connect(self.Update)
    self.gradient_edit_.gradientUpdated.connect(self.Update)
    self.gradient_preset_.gradientSelected.connect(self.gradient_edit_.LoadGradient)
    
    self.setMinimumSize(250,300)
        
  def Update(self):
    scene_selection = gui.SceneSelection.Instance()
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      self.ChangeColor(node)
      
    if(scene_selection.GetActiveViewCount() > 0):
      entity = scene_selection.GetViewEntity()
      view = scene_selection.GetViewUnion()
      self.ChangeViewColor(entity,view)
  
  def ChangeColor(self,node):
    if isinstance(node, gfx.Entity) or isinstance(node, gfx.Surface):
      node.CleanColorOps()
      if self.parent_.GetCarbonsOnly():
        node.ColorBy(self.props[self.prop_combo_box_.currentIndex()],
                     self.gradient_edit_.GetGfxGradient(), "ele=C")
      else:
        node.ColorBy(self.props[self.prop_combo_box_.currentIndex()],
                     self.gradient_edit_.GetGfxGradient())

  def ChangeViewColor(self, entity, view):
    if isinstance(entity, gfx.Entity) and isinstance(view, mol.EntityView):
      if self.parent_.GetCarbonsOnly():
        glco=gfx.GradientLevelColorOp(mol.QueryViewWrapper(mol.Query("ele=C"),view),self.props[self.prop_combo_box_.currentIndex()],self.gradient_edit_.GetGfxGradient(),mol.Prop.Level.UNSPECIFIED)
      else:
        glco=gfx.GradientLevelColorOp(mol.QueryViewWrapper(view),self.props[self.prop_combo_box_.currentIndex()],self.gradient_edit_.GetGfxGradient(),mol.Prop.Level.UNSPECIFIED)
      entity.Apply(glco)
      
#Gradient Preview
class GradientPreview(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Defaults
    self.border_offset_ = 3
    self.preview_height_ = 25
    QtWidgets.QWidget.__init__(self, parent)
    
    self.gradient_ = QtGui.QLinearGradient()
    
    #Ui
    self.setMinimumSize(0, self.preview_height_ + 4)

  def SetGradient(self, gradient):
    self.gradient_ = gradient
    self.update()

  def paintEvent(self, event):   
    if self.isEnabled() and self.gradient_ is not None:
      size = self.size()
      paint = QtGui.QPainter()
      if paint.begin(self):
        brush = QtGui.QBrush(self.gradient_)
        paint.setBrush(brush)
        paint.drawRect(self.border_offset_,
                       self.border_offset_,
                       size.width() - 2 * self.border_offset_,
                       self.preview_height_)
        paint.end()
    
#Gradient Edit
class GradientEdit(QtWidgets.QWidget):
  
  gradientUpdated = QtCore.pyqtSignal()

  def __init__(self, gradient_preview, parent=None):
    QtWidgets.QWidget.__init__(self, parent)

    #Defaults
    self.gradient_preview_ = gradient_preview
    self.border_offset_ = self.gradient_preview_.border_offset_
    self.edit_height_ = 20
        
    #Stops
    self.stops = list()
    
    
    #ContextMenu    
    self.add_ = QtWidgets.QAction("Add", self)
    self.add_.triggered.connect(self.Add)
    self.addAction(self.add_)
    self.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
    
    #Ui
    pal = QtGui.QPalette(self.palette())
    pal.setColor(QtGui.QPalette.Window, QtGui.QColor(255, 255, 255, 128))
    self.setPalette(pal)
    self.setAutoFillBackground(True) 
    self.gradient_ = QtGui.QLinearGradient()
    
    self.setMinimumSize(0, self.edit_height_)
    
    #Add Points
    self.AddStop(self.border_offset_+3, QtGui.QColor("Red"))
    self.AddStop(self.width()-(self.border_offset_+3), QtGui.QColor("Green"))

    #Width Problem
    self.width_ = self.width()

  def RemoveStop(self, stop):
    self.stops.remove(stop)
    stop.hide()
    del(stop)

    self.UpdateGradient()
    self.gradientUpdated.emit()
      
  def RemoveStopGui(self, stop):
    if(len(self.stops)>2):
      self.RemoveStop(stop)
    else:
      QtWidgets.QMessageBox.question(self, "Information", "Please keep in mind, at least two stops are needed for a gradient!")

  def AddStop(self, pos, color=None):
    stop = MyGradientStop(pos, self.border_offset_, self, color)
    stop.gradientChanged.connect(self.UpdateGradient)
    stop.colorChanged.connect(self.UpdateGradient)
    stop.colorChanged.connect(self.parent().Update)
    stop.gradientUpdated.connect(self.parent().Update)
    stop.removeStop.connect(self.RemoveStopGui)
    stop.MoveToNewPos()
    
    self.stops.append(stop)
    self.UpdateGradient() 
    self.gradientUpdated.emit()
  
  def Add(self):
    self.AddStop(QtWidgets.QCursor.pos().x() - self.mapToGlobal(QtCore.QPoint(0, 0)).x())
  
  def UpdateGradient(self):
    self.gradient_preview_.SetGradient(self.GetGradient())      

  def GetGfxGradient(self):  
    gradient = gfx.Gradient()
    for s in self.stops:
      c=s.GetColor();
      gradient.SetColorAt(s.GetRel(), gfx.RGB(c.redF(), c.greenF(), c.blueF()));
    return gradient;

  def GetGradient(self):
    gradient = QtGui.QLinearGradient(self.border_offset_, 
                                     0,
                                     self.size().width() - 2 * self.border_offset_, 
                                     0)
    for s in self.stops:
      gradient.setColorAt(s.GetRel(), s.GetColor())
    return gradient

  def LoadGradient(self, gradient):
    stopcpy = self.stops[:]
    for stop in stopcpy:
      self.RemoveStop(stop)
    
    gradStops = gradient.stops()    
    for stop in gradStops:
      self.AddStop((self.width()-(2*self.border_offset_))*stop[0], stop[1])
  
  #Overwritten methods
  def mouseDoubleClickEvent(self, event):
    self.AddStop(event.x())

  def resizeEvent(self, event):
    factor = 1/(float(self.width_)/event.size().width())
    if(factor >=0):
      for s in self.stops:
        s.SetPos(s.GetPos() * factor)
      self.UpdateGradient()
      self.width_ = event.size().width()
    
#Gradient Stop  
class MyGradientStop(ColorSelectWidget):

  removeStop = QtCore.pyqtSignal()
  gradientChanged = QtCore.pyqtSignal()
  gradientUpdated = QtCore.pyqtSignal()

  def __init__(self, pos, offset, parent, color=None):
    #Defaults
    self.length_ = 20
    self.halflength_ = self.length_ / 2
    
    self.pos_ = pos
    self.offset_ = offset
    
    ColorSelectWidget.__init__(self, self.length_,self.length_, color, parent)
    
    if(color is None):
      self.color_ = QtGui.QColor("White")
      self.ChangeColor()
    else:
      self.color_ = color
    
    #ContextMenu    
    self.remove_ = QtWidgets.QAction("Remove", self)
    self.remove_.triggered.connect(self.Remove)
    self.addAction(self.remove_)
    
  def Remove(self):
    self.removeStop.emit()

  def GetPos(self):
    return self.pos_
  
  def SetPos(self, pos):
    self.pos_ = pos
    self.MoveToNewPos()
    
  def GetRel(self):
    rel = self.pos_ / float(self.parent().width() - 2 * self.offset_)
    if rel > 1:
      return 1
    elif rel < 0:
      return 0
    return rel
  
  def MoveToNewPos(self):
    self.move(self.pos_ - self.halflength_, 0)
    self.update()
    self.gradientChanged.emit()
 
  def mouseMoveEvent(self, event):
    self.pos_ += event.pos().x() - self.halflength_
    if self.pos_ <= self.offset_:
      self.pos_ = self.offset_ + 1
    elif self.pos_ >= self.parent().width() - 2*self.offset_:
      self.pos_ = self.parent().width() - (2*self.offset_ + 1)
    self.MoveToNewPos()
    
  def mouseDoubleClickEvent(self, event):    
    self.ChangeColor()
    
  def mouseReleaseEvent(self, entity):
    if entity.button() == QtCore.Qt.LeftButton:
      self.gradientUpdated.emit()



import os
import subprocess
import shutil
import sys
import glob

def _lib_name(component):
  return 'libost_%s.dylib' % component

def _deps_for_lib(lib, pool, recursive=True):
  if lib in pool:
    return
  otool=subprocess.Popen(['otool', '-L', lib], stdout=subprocess.PIPE)
  output=otool.communicate()[0]
  lines=output.split('\n')[1:]
  for line in lines:
    d=line.split(' ')[0].strip()
    if len(d)>0:
      if d==lib:
        continue
      if d not in pool:
        if d.startswith('/System') or d.startswith('/usr/lib'):
          continue
        if recursive:
          _deps_for_lib(d, pool)
        pool.add(d)        
  return

def collect_deps(stage_dir, components, binaries, libexec_binaries,
                 site_packages, site_packages_dir, libexec_path='openstructure'):
  """
  Collect the dependencies for the given components and returns a list of 
  frameworks/libraries that the component depends on.
  """
  pool=set()
  for component in components:
    lib_name=os.path.abspath(os.path.join(stage_dir, 'lib', 
                                          _lib_name(component)))  
    if not os.path.exists(lib_name):
      print('WARNING:', lib_name, 'does not exist')
    if lib_name not in pool:
      _deps_for_lib(lib_name, pool)
      pool.add(lib_name)    
  for bin in binaries:  
    bin_name=os.path.abspath(os.path.join(stage_dir, 'bin', 
                                          bin))  
    if not os.path.exists(bin_name):
      print('WARNING:', bin_name, 'does not exist')
      continue
    if bin_name not in pool:
      _deps_for_lib(bin_name, pool)
  for bin in libexec_binaries:
    bin_name=os.path.abspath(os.path.join(stage_dir, 'libexec/openstructure',
                                          bin))
    if not os.path.exists(bin_name):
      print('WARNING:', bin_name, 'does not exist')
      continue
    if bin_name not in pool:
      _deps_for_lib(bin_name, pool)
  for site_package in site_packages:
    full_path=get_python_module_path(site_package)
    print(full_path)
    if not os.path.exists(full_path):
      print('WARNING:', site_package, 'does not exists')
      continue
    if os.path.isdir(full_path):
      for so_file in glob.glob(os.path.join(full_path, '*.so')):
        _deps_for_lib(so_file, pool)
  return pool

LIBEXEC_SCRIPTS=['ost_config']
LIBEXEC_BINARIES=[]
GUI_LIBEXEC_BINARIES=['gosty']
BINARIES=['lddt', 'chemdict_tool', 'tmalign', 'tmscore']
GUI_BINARIES=[]
GUI_COMPONENTS=['gfx', 'gui', 'info']
COMPONENTS=['mol', 'geom', 'conop', 'seq_alg', 'seq',
            'img', 'img_alg', 'io', 'db', 'base']
GUI_SCRIPTS=['dng']
SCRIPTS=['ost']
CHANGE_ID_RPATH='install_name_tool -id @rpath/%s %s'   
CHANGE_ID='install_name_tool -id @rpath/%s %s'
CHANGE_LOAD_CMD_RPATH='install_name_tool -change %s @rpath/%s %s'
CHANGE_LOAD_CMD='install_name_tool -change %s @executable_path/%s %s'
ADD_RPATH='install_name_tool -add_rpath %s %s 2> /dev/null'
SITE_PACKAGES=[]
GUI_SITE_PACKAGES=['sip.so', 'sipconfig.py', 'sipdistutils.py', 'PyQt4']
REMOVE_HEADERS='rm -rf `find %s/lib -type d -name Headers`'
REMOVE_CURRENT='rm -rf `find %s/lib -type d -name Current`'
# collect libs of non-standard libraries/frameworks we depend on

def copy_binaries(stage_dir, outdir, binary_names, scripts, bin_dir,
                  append_bin=True):

  exe_path=os.path.abspath(os.path.join(outdir, bin_dir))
  for binary_name in binary_names:
    if append_bin:
      bin_name=os.path.join(stage_dir, bin_dir, binary_name)
    else:
      bin_name=os.path.join(stage_dir, binary_name)
    if not os.path.exists(bin_name):
      print('WARNING:', binary_name, 'does not exist')
      continue
    dst_name=os.path.join(outdir, bin_dir, os.path.basename(bin_name))
    shutil.copy(bin_name, dst_name)
    update_load_commands(dst_name, True, exe_path)
  for script in scripts:
    shutil.copy(os.path.join(stage_dir, bin_dir, script),
                os.path.join(outdir,bin_dir, script))

def split_framework_components(abs_path):
    """
    Splits the path pointing to a dynamic library within a framework
    
    '/System/Frameworks/A.framework/Versions/4/A' =>
    ['/System/Frameworks/A.framework', 'Versions/4/A']
    """
    parts=abs_path.split('/')
    for i, s in enumerate(parts):
      if s.endswith('.framework'):
        lead=os.path.join('/', *parts[:i+1])
        trail=os.path.join(*parts[i+1:])
        return lead, trail

def change_id(id, lib):
  os.chmod(lib, 0o666)
  os.system(CHANGE_ID_RPATH % (id,lib))
  os.chmod(lib, 0o444)

def update_load_commands(lib, exe, exe_path):
  direct_deps=set()
  _deps_for_lib(lib, direct_deps, recursive=False)
  os.chmod(lib, 0o666)
  for direct_dep in direct_deps:
    if direct_dep.endswith('.dylib'):
      new_name=os.path.basename(direct_dep)
      os.system(CHANGE_LOAD_CMD_RPATH % (direct_dep, new_name, lib))
    else:
      assert direct_dep.find('.framework/')>=0
      framework_path, rel_path=split_framework_components(direct_dep)
      framework_name=os.path.basename(framework_path)
      new_name=os.path.join(framework_name, rel_path)
      os.system(CHANGE_LOAD_CMD_RPATH % (direct_dep, new_name, lib))
  if exe:
    os.chmod(lib, 0o555)
  else:
    os.chmod(lib, 0o444)

def copy_deps(dependencies, outdir):
  exe_path=os.path.join(outdir, 'bin')
  for dep in dependencies:
    if dep.endswith('.dylib'):
      dst_name=os.path.join(outdir, 'lib', os.path.basename(dep))
      if not os.path.exists(dep):
        continue
      shutil.copy(dep, dst_name)
      change_id(os.path.basename(dep), dst_name)
      update_load_commands(dst_name, False, exe_path)
    else:
      assert dep.find('.framework/')>=0
      framework_path, rel_path=split_framework_components(dep)
      framework_name=os.path.basename(framework_path)
      dst_name=os.path.join(outdir, 'lib', framework_name)
      shutil.copytree(framework_path, dst_name)
      change_id(os.path.join(dst_name, rel_path),
                os.path.join(dst_name, rel_path))
      os.unlink(os.path.join(dst_name, os.path.splitext(framework_name)[0]))
      update_load_commands(os.path.join(dst_name, rel_path), False, 
                           exe_path)

def update_pymod_shared_objects(lib_path, path, files):
  exe_path=os.path.abspath(os.path.join(lib_path, '../bin'))
  for f in files:
    if not os.path.exists(os.path.join(path, f)):
      continue
    base, ext=os.path.splitext(f)
    if  ext=='.so':
      path_to_lib_path=os.path.relpath(lib_path, path)
      abs_name=os.path.join(path, f)
      os.system(ADD_RPATH % (path_to_lib_path, abs_name))
      update_load_commands(abs_name, False, exe_path)
    elif ext in ('.pyc', '.pyo'):
      os.unlink(os.path.join(path, f))

def get_site_package_dir():
  """
  Get site-package directory of this python installation. This assumes 
  that ost was linked against the same version of Python
  """
  for p in sys.path:
    pattern='/site-packages'
    index=p.find(pattern)
    if index>=0:
      return p[:index+len(pattern)]
  raise RuntimeError("Couldn't determine site-packages location")

def get_python_module_path(module):
  for path in sys.path:
    full_path=os.path.join(path, module)
    if os.path.exists(full_path):
      return full_path
  return None
  
  
def get_python_home():
  """
  Derive Python home by looking at the location of the os module
  """
  return os.path.dirname(sys.modules['os'].__file__)

def make_standalone(stage_dir, outdir, no_includes, force_no_rpath=False,
                    macports_workaround=False, no_gui=False):
  site_packages_dir=get_site_package_dir()

  if os.path.exists(outdir):
    shutil.rmtree(outdir)
  os.system('mkdir -p "%s"' % outdir)
  os.system('mkdir -p "%s/lib"' % outdir)
  os.system('mkdir -p "%s/bin"' % outdir)
  os.system('mkdir -p "%s/libexec/openstructure"' % outdir)
  print('copying shared datafiles')
  shutil.copytree(os.path.join(stage_dir, 'share'), 
                  os.path.join(outdir, 'share'))
  scripts=SCRIPTS
  binaries=BINARIES
  components=COMPONENTS
  libexec_scripts=LIBEXEC_SCRIPTS
  site_packages=SITE_PACKAGES
  libexec_binaries=LIBEXEC_BINARIES
  if not no_gui:
    scripts+=GUI_SCRIPTS
    binaries+=GUI_BINARIES
    components+=GUI_COMPONENTS
    libexec_binaries+=GUI_LIBEXEC_BINARIES
    site_packages+=GUI_SITE_PACKAGES
  print('collecting dependencies')
  deps=collect_deps(stage_dir, components, binaries, libexec_binaries,
                    site_packages, site_packages_dir)
  # when running in non-gui mode, we are most likely missing the boost
  # python library. Let's add it to the list of dependencies by
  # inspecting "_ost_base.so".
  pymod_dir='lib/python%d.%d/site-packages' % sys.version_info[0:2]
  _deps_for_lib(os.path.join(stage_dir, pymod_dir, 'ost/_ost_base.so'),
                deps, recursive=False)
  print('copying dependencies')
  copy_deps(deps, outdir)
  print('copying libexec binaries')
  copy_binaries(stage_dir, outdir, libexec_binaries, libexec_scripts,
                'libexec/openstructure')
  print('copying binaries')
  copy_binaries(stage_dir, outdir, binaries, scripts, 'bin')
  print('copying pymod')
  shutil.copytree(os.path.join(stage_dir,pymod_dir),
                  os.path.join(outdir, pymod_dir))
  copied_py_framework=False
  non_std_python=False
  if os.path.exists(os.path.join(outdir, 'lib/Python.framework')):
    framework_path=os.path.join(outdir, 'lib/Python.framework')
    nonstd_python=True
    copied_py_framework=True
  if len(glob.glob(os.path.join(outdir, 'lib', 'libpython*')))>0:
    non_std_python=True
  if non_std_python:
    print('looks like we are using a non-standard python.')
    python_home=get_python_home()    
    if not copied_py_framework:
      print('also copying python modules from %s' % python_home)
      modules_dst=os.path.join(outdir, 'lib', os.path.basename(python_home))
      shutil.copytree(python_home, modules_dst)
      if os.path.exists(os.path.join(modules_dst, 'site-packages')):
        shutil.rmtree(os.path.join(modules_dst, 'site-packages'))
      copy_binaries(os.path.join(python_home, '../..'), outdir, 
                    ['python'], [], 'bin')
      python_bin=os.path.abspath(os.path.join(python_home, '../../bin/python'))
    else:
      # For MacPorts it's even more involved. Python is not executed directly 
      # but rather uses a wrapper executable that calls the actual python exe.
      # We have to include that one into the bundle.
      if macports_workaround:
        path_to_app='../../Resources/Python.app/Contents/MacOS/'
        exe_path=os.path.join(python_home, path_to_app)
        copy_binaries(exe_path, outdir, ['python'], [], 
                      append_bin=False)
        python_bin=os.path.join('/opt/local/bin/python')
      else:
        copy_binaries(os.path.join(python_home, '../..'), outdir, 
                      ['python'], [])
        python_bin=os.path.abspath(os.path.join(python_home, '../../bin/python'))
      # remove all versions but the one we are using
      version_string=sys.version[0:3]
      prefix, postfix=split_framework_components(python_home)
      site_packages_dir=os.path.join(outdir, 'lib', 'Python.framework', 
                                 postfix, 'site-packages')
      if os.path.exists(site_packages_dir):
        shutil.rmtree(site_packages_dir)
      for directory in glob.glob(os.path.join(framework_path, 'Versions/*')):
        if os.path.basename(directory)!=version_string:
          shutil.rmtree(directory)
    # replace the python executable
    ost_script=os.path.join(outdir, 'libexec', 'openstructure', 'ost_config')
    os.chmod(ost_script, 0o666)
    script=''.join(open(ost_script, 'r').readlines())
    script=script.replace(python_bin, '$BIN_DIR/python')
    open(ost_script, 'w').write(script)
    os.chmod(ost_script, 0o555)

  if no_includes:
    os.system(REMOVE_HEADERS % outdir)
    os.system(REMOVE_CURRENT % outdir)  
  print('copying site-packages')
  for sp in SITE_PACKAGES:
    src=get_python_module_path(sp)
    if os.path.isdir(src):
      shutil.copytree(src, os.path.join(outdir, pymod_dir, sp))
    else:
      shutil.copy(src, os.path.join(outdir, pymod_dir, sp))
  print('updating link commands of python shared objects')
  os.path.walk(os.path.join(outdir, 'lib'), 
               update_pymod_shared_objects, 
               os.path.join(outdir, 'lib'))



import math

def Mean(xs):
  """
  Calculate mean of dataset
  """
  if len(xs)==0:
    raise RuntimeError("Can't calculate mean of empty sequence")
  return float(sum(xs))/len(xs)

def Median(xs):
  """
  Calculate median of dataset
  """
  if len(xs)==0:
    raise RuntimeError("Can't calculate median of empty sequence")
  sorted_xs=sorted(xs)
  central_idx = int((len(xs)-1)/2) 
  if (len(xs) % 2)==0:
    return (sorted_xs[central_idx]+sorted_xs[central_idx+1])/2.0
  else:
    return sorted_xs[central_idx]

def StdDev(xs):
  """
  Calculate standard-deviation of dataset
  
            | sum[xi-<x>]^2 |
  sigma=sqrt|---------------|
            |       n       |
  """
  mean=Mean(xs)
  return math.sqrt(sum([(x-mean)**2 for x in xs])/len(xs))

def Min(xs):
  return min(xs)

def Max(xs):
  return max(xs)

def Correl(xs, ys):
  """
  Calculates the correlation coefficient between xs and ys as
  
    sum[(xi-<x>)*(yi-<y>)]
  r=----------------------
          sx*sy
          
  where <x>, <y> are the mean of dataset xs and ys, and, sx and sy are the 
  standard deviations.
  """
  if len(xs)!=len(ys):
    raise RuntimeError("Can't calculate correl. Sequence lengths do not match.")
  if len(xs)==1:
    raise RuntimeError("Can't calculate correl of sequences with length 1.")
  mean_x=Mean(xs)
  mean_y=Mean(ys)
  sigma_x, sigma_y=(0.0, 0.0)
  cross_term=0.0
  for x, y in zip(xs, ys):
    cross_term+=(x-mean_x)*(y-mean_y)
    sigma_x+=(x-mean_x)**2
    sigma_y+=(y-mean_y)**2
  sigma_x=math.sqrt(sigma_x)
  sigma_y=math.sqrt(sigma_y)
  return cross_term/(sigma_x*sigma_y)

def Histogram(xs, bounds, num_bins):
  bins=[0 for i in range(num_bins)]
  d=1.0*num_bins/(bounds[1]-bounds[0])
  for x in xs:
    index=int((x-bounds[0])*d)
    if index>num_bins-1 or index<0:
      continue
    bins[index]+=1
  return bins



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
#
# Author: Ansgar Philippsen
#
# automatically generate iso-contouring marching cube specs
#

import math

# cube definition

# corners:
#     5        6
#      -------
#     /.     /|
#    / .    / |
#  1 ------ 2 |
#   |  ....|..|
#   | .4   | / 7
#   |.     |/
#    ------
#  0        3

# vertices:              
#      ---9---
#     5.     6|
#    / 8    / |
#    ---1--   10
#   |  ..11|..|
#   0 4    2 7  
#   |.     |/
#    --3---

# faces     2
#          .       
#      -------
#     /      /|
#    /   1  / |
#    ------  .|.. 5
#4..|      |  |
#   |  0   | / 
#   |      |/.
#    ------   3

corner_coord=[[0,0,0],[0,1,0],[1,1,0],[1,0,0],
               [0,0,1],[0,1,1],[1,1,1],[1,0,1]]

edge_corner=[[0,1],[1,2],[2,3],[3,0],
                [0,4],[1,5],[2,6],[3,7],
                [4,5],[5,6],[6,7],[4,7]]

face_edges=[[0,1,2,3],[1,5,9,6],[9,8,11,10],
            [11,7,3,4],[4,8,5,0],[7,2,6,10]]

header_code="""
// automatically generated by generate_map_iso_spec.py
// do not edit directly

#include "map_iso_gen.hh"

namespace ost { namespace gfx { namespace map_iso {

template<int N>
void AddLinesAndFaces(IndexedVertexArray& va, // vertex array
		      unsigned int vertex_id[12] // this list of vertex ids
		      );
"""

isocube_header_code="""
template<> 
void AddLinesAndFaces<%d>(IndexedVertexArray& va,unsigned int vertex_id[12] )
{
"""

isocube_addline_code="""  va.AddLine(vertex_id[%d],vertex_id[%d]);
"""

isocube_addface_code="""  va.AddTriN(vertex_id[%d],vertex_id[%d],vertex_id[%d]);
"""

isocube_debug_code="""
#ifdef MAP_ISO_DEBUG
  if(vertex_id[%d]==0) std::cerr << \"ISO_DEBUG: \" << %d << \" \" << %d << std::endl;
  if(vertex_id[%d]==0) std::cerr << \"ISO_DEBUG: \" << %d << \" \" << %d << std::endl;
  if(vertex_id[%d]==0) std::cerr << \"ISO_DEBUG: \" << %d << \" \" << %d << std::endl;
#endif
"""

isocube_footer_code="}\n"

flist_header_code="""
void GenerateLFList(AddLFList& lfl)
{
  lfl.clear();
"""

flist_code="  lfl.push_back(AddLinesAndFaces<%d>);\n"

flist_footer_code="""
}
"""

footer_code="""
}}} // ns
"""

def vadd(v1,v2):
    return [v1[0]+v2[0],
            v1[1]+v2[1],
            v1[2]+v2[2]]

def vsub(v1,v2):
    return [v1[0]-v2[0],
            v1[1]-v2[1],
            v1[2]-v2[2]]

def vmul(s,v1):
    return [v1[0]*s,
            v1[1]*s,
            v1[2]*s]

def vdot(v1,v2):
    return v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]

def vcross(v1,v2):
    return [v1[1]*v2[2]-v2[1]*v1[2],
	    v1[2]*v2[0]-v2[2]*v1[0],
	    v1[0]*v2[1]-v2[0]*v1[1]]

def vnorm(v1):
    len2=v1[0]*v1[0]+v1[1]*v1[1]+v1[2]*v1[2]
    return vmul(1.0/math.sqrt(len2),v1)

def work(spec_file,pattern):

    corner_bit=[0,0,0,0,0,0,0,0]
    for i in range(8):
        if pattern & (1<<i):
            corner_bit[i]=1

    edge_bit=[0,0,0,0,0,0,0,0,0,0,0,0]
    edge_list=[]
    for i in range(12):
        if corner_bit[edge_corner[i][0]] != corner_bit[edge_corner[i][1]]:
            edge_bit[i]=1
            edge_list.append(i)

    spec_file.write(isocube_header_code%pattern)

    # first the lines
    edge_pairs=[]
    for e1 in edge_list:
        for e2 in edge_list:
            if e1>e2:
                for f in face_edges:
                    if e1 in f and e2 in f:
                        # line across a face
                        edge_pairs.append([e1,e2])
                        spec_file.write(isocube_addline_code%(e1,e2))

    # find line-loop(s)
    while len(edge_pairs)>0:
        edge_loop=[]
        edge_loop.extend(edge_pairs[0])
        edge_pairs.remove(edge_pairs[0])
        closed=False
        while not closed:
            for ep in edge_pairs:
                if ep[0]==edge_loop[len(edge_loop)-1]:
                    if ep[1]==edge_loop[0]:
                        closed=True
                    else:
                        edge_loop.append(ep[1])
                    edge_pairs.remove(ep)
                    break
                elif ep[1]==edge_loop[len(edge_loop)-1]:
                    if ep[0]==edge_loop[0]:
                        closed=True
                    else:
                        edge_loop.append(ep[0])
                    edge_pairs.remove(ep)
                    break

        # find correct orientation 
        # assemble dir0 from middle of edge and the higher value corner
        edge_id0 = edge_loop[0]
        v00 = edge_corner[edge_id0][0]
        v01 = edge_corner[edge_id0][1]
        e0_pos = vmul(0.5,vadd(corner_coord[v00],corner_coord[v01]))
        e1_pos=[0,0,0]
        if corner_bit[v00]==1:
            e1_pos=corner_coord[v00]
        else:
            e1_pos=corner_coord[v01]
        dir0 = vnorm(vsub(e1_pos,e0_pos))
        # assemble dir1 from the cross product of the lines of the
        # edge_loop
        edge_id1 = edge_loop[1]
        v10 = edge_corner[edge_id1][0]
        v11 = edge_corner[edge_id1][1]
        edge_id2 = edge_loop[2]
        v20 = edge_corner[edge_id2][0]
        v21 = edge_corner[edge_id2][1]
        e1_pos = vmul(0.5,vadd(corner_coord[v10],corner_coord[v11]))
        e2_pos = vmul(0.5,vadd(corner_coord[v20],corner_coord[v21]))
        e10 = vsub(e0_pos,e1_pos)
        e12 = vsub(e2_pos,e1_pos)
        dir1 = vnorm(vcross(e10,e12))
        # this is the actual orientation check
        if vdot(dir0,dir1)<0.0:
            edge_loop.reverse()
        # dump code to file
        for i in range(len(edge_loop)-2):
            spec_file.write(isocube_addface_code%(edge_loop[0],edge_loop[i+2],edge_loop[i+1]))
            spec_file.write(isocube_debug_code%(edge_loop[0],edge_loop[0],pattern,
                                                edge_loop[i+2],edge_loop[i+2],pattern,
                                                edge_loop[i+1],edge_loop[i+1],pattern))
                                                
            
    spec_file.write(isocube_footer_code)
    

spec_file=file("map_iso_spec.hh","w")

spec_file.write(header_code)
    
for i in range(256):
    work(spec_file,i)

spec_file.write(flist_header_code)
for i in range(256):
    spec_file.write(flist_code%i)
spec_file.write(flist_footer_code)

spec_file.write(footer_code)



from PyQt5 import QtWidgets
from ost import gui
class DocWidget(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    l=QtWidgets.QVBoxLayout(self)
    l.setContentsMargins(0,0,0,0)
    self.setWindowTitle('OpenStructure Help')
    self.searchbox=QtWidgets.QLineEdit(self)
    l.addWidget(self.searchbox)
    self.doctext=QtWidgets.QTextEdit('', self)
    l.addWidget(self.doctext)
    self.setLayout(l)
    self.searchbox.returnPressed.connect(self.OnReturnPressed)
  def OnReturnPressed(self):
    help(str(self.searchbox.text()))
doc_widget=DocWidget()
doc_widget_for_panel=gui.Widget(doc_widget)
panels=gui.GostyApp.Instance().perspective.panels
panels.AddWidgetToPool("OpenStructure Help",doc_widget_for_panel)


class Help:
  def __call__(self, what=None):
    import pydoc
    d=pydoc.HTMLDoc()
    if what==None:
      doc_widget.doctext.setHtml('')
      doc_widget.searchbox.setText('')
    else:
      try:
        s=d.document(pydoc.resolve(what)[0])
      except ImportError as detail:
        s=str(detail)
      doc_widget.doctext.setHtml(s)
      doc_widget.searchbox.setText('')
    if not doc_widget.isVisible():
      panels.AddWidget(gui.PanelPosition.RIGHT_PANEL, doc_widget_for_panel, False)
  def __repr__(self):
    return 'Type help() for interactive help, or help(object) for help about object.'

help=Help()



# This script shows how to switch between different render modes 
# programmatically and change render options.


# remove all objects from scene, just in case
scene.RemoveAll()

# load pdb structure
eh=io.LoadPDB("data/sdh.pdb")


sdh_go=gfx.Entity("SDH2", eh.Select("cname=A"))
# create graphical object and add it to the scene
scene.Add(sdh_go)
scene.CenterOn(sdh_go)

def Tube():
  sdh_go.SetRenderMode(gfx.TUBE)
  sdh_go.tube_options.SetTubeRadius(0.8)
  # apply color gradient for the atomic bfactors
  sdh_go.ColorBy("abfac",gfx.BLUE,gfx.RED)
  # and apply radius gradient as well 
  sdh_go.RadiusBy("abfac",0.8,2.2)
  
def Trace():
  sdh_go.SetRenderMode(gfx.LINE_TRACE)
  sdh_go.GetOptions(gfx.LINE_TRACE).SetLineWidth(2.5)

def Spheres():
  sdh_go.SetRenderMode(gfx.CPK)
  sdh_go.GetOptions(gfx.CPK).SetSphereDetail(4)

def Cartoon():  
  sdh_go.SetRenderMode(gfx.RenderMode.HSC)
  sdh_go.SetColor(gfx.Color(0.5, 0.5, 0.5), '')
  sdh_go.SetColor(gfx.Color(0,1,0),"rtype=H")
  sdh_go.SetDetailColor(gfx.Color(0.7,1.0,0.8),"rtype=H")
  # strands
  sdh_go.SetColor(gfx.Color(1,0,0),"rtype=E")
  sdh_go.SetDetailColor(gfx.Color(1.0,0.8,0.2),"rtype=E")
  # these are the default params for the rendering
  #sdh_go.cartoon_options.SetArcDetail(4)    # circular profile detail
  #sdh_go.cartoon_options.SetSphereDetail(4) # sphere detail
  #sdh_go.cartoon_options.SetSplineDetail(8) # segments between backbone atoms
  #sdh_go.cartoon_options.SetTubeRadius(0.4) # coil radius 
  #sdh_go.cartoon_options.SetTubeRatio(1.0)  # coil axial ratio
  #sdh_go.cartoon_options.SetHelixWidth(1.1)   # helical width
  #sdh_go.cartoon_options.SetHelixThickness(0.2)   # helical thickness
  #sdh_go.cartoon_options.SetStrandWidth(1.2)  # strand width
  #sdh_go.cartoon_options.SetStrandThickness(0.2)  # strand thickness

Cartoon()




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

from ost import gui
from ost import gfx
import ost
import os
from datetime import datetime
from PyQt5 import QtCore, QtWidgets, QtGui
from .scene_selection_helper import SelHelper
from .preset_list_model import PresetListModel
from .preset_editor_widget import PresetEditor
from .preset import Preset

class PresetWidget(QtWidgets.QWidget):
  PRESET_XML_FILE = os.path.join(ost.GetSharedDataPath(), "scene", "presets.xml")
  ICONS_DIR = os.path.join(ost.GetSharedDataPath(), "gui", "icons/")
  def __init__(self, parent=None):   
    QtWidgets.QWidget.__init__(self, parent)
    
    self.text_ = "Presets"
        
    #Create Ui elements
    self.list_view_ = QtWidgets.QListView()
        
    #Create Model
    self.list_model_ = PresetListModel(self)
    self.list_view_.setModel(self.list_model_)
    self.list_view_.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
    
    self.add_action = QtWidgets.QAction("+",self)
    self.add_action.setIcon(QtGui.QIcon(PresetWidget.ICONS_DIR+"add_icon.png"))
    
    self.add_action.triggered.connect(self.Add)
    
    self.add_button_ = QtWidgets.QToolButton(self)
    self.add_button_.setIconSize(QtCore.QSize(20,20))
    self.add_button_.setDefaultAction(self.add_action)
    
    self.preset_editor_ = PresetEditor(self)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.list_view_,0,0,3,3)
    grid.addWidget(self.add_button_,3,0,1,1)
    self.setLayout(grid)
    
    self.list_view_.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
    self.list_view_.customContextMenuRequested.connect(self.contextMenuEvent)
    self.CreateImmutableContextMenu()
    self.CreateContextMenu()
  
    self.list_view_.doubleClicked.connect(self.Load)
    
    self.setMinimumSize(250,200)
    
  def CreateImmutableContextMenu(self):  
    self.immucontext_menu_ = QtWidgets.QMenu("Context menu", self)
    self.load_ = QtWidgets.QAction("Load", self.list_view_)  
    self.immucontext_menu_.addAction(self.load_)
    #Connect Signal with Slot  
    self.load_.triggered.connect(self.LoadCurrentIndex)

  def CreateContextMenu(self):
    self.context_menu_ = QtWidgets.QMenu("Context menu", self)
    self.remove_ = QtWidgets.QAction("Remove", self.list_view_)
    self.rename_ = QtWidgets.QAction("Rename", self.list_view_)
    self.edit_ = QtWidgets.QAction("Edit", self.list_view_)
    self.context_menu_.addAction(self.load_)
    self.context_menu_.addAction(self.remove_)
    self.context_menu_.addAction(self.rename_)
    self.context_menu_.addAction(self.edit_)
    #Connect Signals with Slots  
    self.remove_.triggered.connect(self.Remove)
    self.rename_.triggered.connect(self.Rename)
    self.edit_.triggered.connect(self.Edit)
  
  def contextMenuEvent(self, pos):
    #ContextMenu
    index = self.list_view_.indexAt(pos)
    if index.isValid(): 
      if self.list_model_.IsEditable(index.row()):
        self.context_menu_.popup(QtWidgets.QCursor.pos())
      else:
        self.immucontext_menu_.popup(QtWidgets.QCursor.pos())
  
  def Add(self):
    row = self.list_model_.GetLastRow()
    preset = Preset(datetime.now().isoformat(' '))
    self.preset_editor_.SetPreset(preset)
    if(self.preset_editor_.exec_()):
      if self.list_model_.AddItem(preset, row, True, True):
        index = self.list_model_.index(row)
        self.list_view_.setCurrentIndex(index)
        self.Rename()
      
  def Remove(self):
    if(self.list_view_.currentIndex().isValid()):
      ret = QtWidgets.QMessageBox.warning(self, "Delete Preset",
                   "Delete Preset?",
                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
      if ret == QtWidgets.QMessageBox.Yes:
        self.list_model_.RemoveItem(self.list_view_.currentIndex().row())

  def Edit(self):
    if(self.list_view_.currentIndex().isValid()):
        preset = self.list_model_.GetPreset(self.list_view_.currentIndex())
        self.preset_editor_.SetPreset(preset)
        if(self.preset_editor_.exec_()):
            row = self.list_view_.currentIndex().row()
            self.list_model_.RemoveItem(row)
            self.list_model_.AddItem(preset, row, True, True)
        
  def LoadCurrentIndex(self):
    if(self.list_view_.currentIndex().isValid()):
      self.Load(self.list_view_.currentIndex())

  def Load(self, index):
    if(index.isValid()):
      scene_selection = gui.SceneSelection.Instance()
      preset=self.list_model_.GetPreset(index)
      for i in range(0,scene_selection.GetActiveNodeCount()):
        node = scene_selection.GetActiveNode(i)
        if isinstance(node, gfx.Entity):
          node.CleanColorOps()
          preset.ApplyOn(node)
          
  def ChangeColor(self,node):
    self.LoadCurrentIndex()
  
  def Update(self):
    self.setEnabled(True)
    if SelHelper().CheckAllFlags(SelHelper.NO_SELECTION):
      self.setEnabled(False)
      return
    
    if SelHelper().CheckNotFlags(SelHelper.HAS_ENTITY | SelHelper.IS_ONE_TYPE):
      self.setEnabled(False)
      return

  def Rename(self):
    if(self.list_view_.currentIndex().isValid()):
      self.list_view_.edit(self.list_view_.currentIndex())
    
  def GetText(self):
    return self.text_



s=io.LoadSurface("../demos/data/sh2","msms")
scene.Add(gfx.Surface("s",s))

# alternative shading mode
scene.SetShadingMode("hf")

# add outlines to surface
scene["s"].SetOutline(True)
scene["s"].SetOutlineMode(3)

# turn on solid rendering
scene["s"].solid=True
scene["s"].solid_color=gfx.RGB(0.8,0.8,0.7)

scene.AutoAutoslab(False)
scene.far=170
scene.near=100
scene.fog=False



"""
Functions to calculate helix kinks: bend, face shift and wobble angles

Author: Niklaus Johner
"""

import os
import ost

def __FindProline(sele,proline):
  if not sele.IsValid():
    print('selection is not valid')
    raise RuntimeError
  if proline==False:
    proline=sele.Select('rname=PRO')
  if not proline.GetResidueCount()==1:
    print(proline.GetResidueCount(),'prolines in the selection. One proline is needed')
    raise RuntimeError
  proline=proline.residues[0]
  proline_ca=proline.FindAtom('CA')
  if not proline_ca.IsValid():
    print('proline has no CA atom')
    raise RuntimeError
  return (proline,proline_ca)
  proline.GetNumber().num
  
def __SelectPreAndPostProline(sele,proline_num):
  pre_proline=sele.Select('rnum<'+str(proline_num))
  post_proline=sele.Select('rnum>'+str(proline_num))
  print('pre-proline residues')
  for res in pre_proline.residues:
    print(res)
  print('post-proline residues')
  for res in post_proline.residues:
    print(res)
  if pre_proline.GetResidueCount()<4 or post_proline.GetResidueCount()<4:
    print('pre and post proline helices should be at least 4 residues long, 7 for better stability')
    raise RuntimeError
  return (pre_proline,post_proline)

def __FindCa3AndCa4(sele,proline_ca,proline_num):
  ca_3=sele.FindAtom(proline_ca.GetHandle().GetChain().GetName(),proline_num-3,'CA')
  ca_4=sele.FindAtom(proline_ca.GetHandle().GetChain().GetName(),proline_num-4,'CA')
  if not (ca_3.IsValid() and ca_4.IsValid()):
    print('CA not found in (i-4) or (i-3) residue')
    raise RuntimeError
  return (ca_3,ca_4)


def __CalculateBendAngle(pre_proline_axis,post_proline_axis):
  return ost.geom.Angle(pre_proline_axis,post_proline_axis)
  
def __CalculateWobbleAngle(pre_proline_axis,post_proline_axis,post_proline_centers,proline_pos):
  p1=proline_pos-post_proline_centers
  n1=p1-ost.geom.Dot(p1,post_proline_axis)*post_proline_axis
  p2=-pre_proline_axis
  n2=p2-ost.geom.Dot(p2,post_proline_axis)*post_proline_axis
  sign=ost.geom.Dot(ost.geom.Cross(pre_proline_axis,n2),n2-n1)
  sign=sign/abs(sign)
  return sign*ost.geom.Angle(n1,n2)

def __CalculateFaceShift(pre_proline_axis,post_proline_axis,pre_proline_centers,post_proline_centers,proline_pos,ca3_pos,ca4_pos,bend_angle):
  p1=proline_pos-post_proline_centers
  n1=p1-ost.geom.Dot(p1,post_proline_axis)*post_proline_axis
  p2=(ca3_pos+ca4_pos)/2.0-pre_proline_centers
  n2=p2-ost.geom.Dot(p2,pre_proline_axis)*pre_proline_axis
  #Here we want to apply a rotation to n1, to align the post-proline axis on the pre-proline axis
  R=ost.geom.AxisRotation(ost.geom.Cross(post_proline_axis,pre_proline_axis),bend_angle)
  n1=R*n1
  #We also need to determine the sign of the angle
  sign=ost.geom.Dot(ost.geom.Cross(pre_proline_axis,n2),n2-n1)
  sign=sign/abs(sign)
  return sign*ost.geom.Angle(n1,n2)


def AnalyzeHelixKink(t, sele, proline=False):
  """
  This function calculates the bend, wobble and face-shift angles in an alpha-
  helix over a trajectory. The determination is more stable if there are at
  least 4 residues on each side (8 is even better) of the proline around which
  the helix is kinked. The selection should contain all residues in the correct
  order and with no gaps and no missing C-alphas.

  :param t: The trajectory to be analyzed
  :type t: :class:`~ost.mol.CoordGroup`
  :param sele: A selection containing the alpha helix to be analyzed
  :type sele: :class:`~ost.mol.EntityView`
  :param proline: A selection containing only the proline (or another residue)
                  around which the helix is kinked. If False, the proline will
                  be searched for automatically
  :type proline: :class:`ost.mol.EntityView`

  :return: A tuple (bend_angle, face_shift, wobble_angle).
  :rtype: (FloatList, FLoatList, FloatList)
  """
  n_frames=t.GetFrameCount()
  (proline,proline_ca)=__FindProline(sele,proline)
  proline_num=proline.GetNumber().num
  (pre_proline,post_proline)=__SelectPreAndPostProline(sele,proline_num)
  (ca_3,ca_4)=__FindCa3AndCa4(sele,proline_ca,proline_num)
  #Here we extract the necessary information from the trajectory
  pre_proline_axis=ost.geom.Vec3List()
  post_proline_axis=ost.geom.Vec3List()
  pre_proline_centers=ost.geom.Vec3List()
  post_proline_centers=ost.geom.Vec3List()
  ost.mol.alg.AnalyzeAlphaHelixAxis(t,pre_proline,pre_proline_axis,pre_proline_centers)
  ost.mol.alg.AnalyzeAlphaHelixAxis(t,post_proline,post_proline_axis,post_proline_centers)
  proline_pos=ost.mol.alg.AnalyzeAtomPos(t,proline_ca.GetHandle())
  ca3_pos=ost.mol.alg.AnalyzeAtomPos(t,ca_3.GetHandle())
  ca4_pos=ost.mol.alg.AnalyzeAtomPos(t,ca_4.GetHandle())
  #Now we calculate the bend angle
  bend_angle=ost.FloatList()
  face_shift=ost.FloatList()
  wobble_angle=ost.FloatList()
  for i in range(n_frames):
    bend_angle.append(__CalculateBendAngle(pre_proline_axis[i],post_proline_axis[i]))
    face_shift.append(__CalculateFaceShift(pre_proline_axis[i],post_proline_axis[i],pre_proline_centers[i],post_proline_centers[i],proline_pos[i],ca3_pos[i],ca4_pos[i],bend_angle[i]))
    wobble_angle.append(__CalculateWobbleAngle(pre_proline_axis[i],post_proline_axis[i],post_proline_centers[i],proline_pos[i]))
  return (bend_angle,face_shift,wobble_angle)


def CalculateHelixKink(sele, proline=False):
  """
  This function calculates the bend, wobble and face-shift angles in an alpha-
  helix of an EntityView. The determination is more stable if there are at least
  4 residues on each side (8 is even better) of the proline around which the
  helix is kinked. The selection should contain all residues in the correct
  order and with no gaps and no missing C-alphas.
  
  :param sele: A selection containing the alpha helix to be analyzed
  :type sele:  :class:`~ost.mol.EntityView`
  :param proline: A selection containing only the proline (or another residue)
                  around which the helix is kinked. If False, the proline will
                  be searched for automatically
  :type proline:  :class:`ost.mol.EntityView`

  :return: A tuple (bend_angle, face_shift, wobble_angle).
  :rtype: (float, float, float)
  """
  (proline,proline_ca)=__FindProline(sele,proline)
  proline_num=proline.GetNumber().num
  (pre_proline,post_proline)=__SelectPreAndPostProline(sele,proline_num)
  (ca_3,ca_4)=__FindCa3AndCa4(sele,proline_ca,proline_num)  
  #Here we extract the necessary information from the structure
  pre_proline_axis=ost.mol.alg.structure_analysis.CalculateHelixAxis(pre_proline)
  post_proline_axis=ost.mol.alg.structure_analysis.CalculateHelixAxis(post_proline)
  prepa=pre_proline_axis.GetDirection()
  prepc=pre_proline_axis.GetOrigin()
  postpa=post_proline_axis.GetDirection()
  postpc=post_proline_axis.GetOrigin()
  #calculate the angles
  bend=__CalculateBendAngle(prepa,postpa)
  wobble=__CalculateWobbleAngle(prepa,postpa,postpc,proline_ca.pos)
  shift=__CalculateFaceShift(prepa,postpa,prepc,postpc,proline_ca.pos,ca_3.pos,ca_4.pos,bend)
  return (bend,shift,wobble)
  
  
  
  



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import os
import ost
from ost import gui
from ost import gfx
from ost import info
from datetime import datetime

from datetime import datetime
from PyQt5 import QtCore, QtWidgets, QtGui
from .gradient_info_handler import GradientInfoHandler
from .gradient_list_model import GradientListModel


#Gradient Preset Widget
class GradientPresetWidget(QtWidgets.QWidget):
  ICONS_DIR = os.path.join(ost.GetSharedDataPath(), "gui", "icons/")

  gradientSelected = QtCore.pyqtSignal(object, name="gradientSelected")

  def __init__(self, gradient_edit, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Title
    self.text_ = "Gradient Presets"
    
    #Refrences
    self.gradient_edit_ = gradient_edit   

    #Create Ui elements
    self.list_view_ = QtWidgets.QListView()
    
    #Create Model
    self.list_model_ = GradientListModel(self)
    self.list_view_.setModel(self.list_model_)
    self.list_view_.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        
    preset_label = QtWidgets.QLabel(self.text_)
    font = preset_label.font()
    font.setBold(True)
    
    self.add_action = QtWidgets.QAction("+",self)
    self.add_action.setIcon(QtGui.QIcon(GradientPresetWidget.ICONS_DIR+"add_icon.png"))
    
    self.add_action.triggered.connect(self.Add)
    
    self.add_button_ = QtWidgets.QToolButton(self)
    self.add_button_.setIconSize(QtCore.QSize(20,20))
    self.add_button_.setDefaultAction(self.add_action)
        
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(preset_label, 0, 0, 1, 1)
    qhbox = QtWidgets.QHBoxLayout()
    grid.addWidget(self.list_view_,1,0,3,3)
    grid.addWidget(self.add_button_,4,0,1,1)
    self.setLayout(grid)
    
    self.list_view_.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
    self.list_view_.customContextMenuRequested.connect(self.contextMenuEvent)
    self.CreateImmutableContextMenu()
    self.CreateContextMenu()

    self.list_view_.doubleClicked.connect(self.Load)

  def CreateImmutableContextMenu(self):  
    self.immucontextMenu_ = QtWidgets.QMenu("Context menu", self)
    self.load_ = QtWidgets.QAction("Load", self.list_view_)  
    self.immucontextMenu_.addAction(self.load_)
    #Connect Signal with Slot  
    self.load_.triggered.connect(self.LoadCurrentIndex)

  def CreateContextMenu(self):
    self.contextMenu_ = QtWidgets.QMenu("Context menu", self)
    self.remove_ = QtWidgets.QAction("Remove", self.list_view_)
    self.rename_ = QtWidgets.QAction("Rename", self.list_view_)
    self.contextMenu_.addAction(self.load_)
    self.contextMenu_.addAction(self.remove_)
    self.contextMenu_.addAction(self.rename_)
    #Connect Signals with Slots  
    self.remove_.triggered.connect(self.Remove)
    self.rename_.triggered.connect(self.Rename)
  
  def contextMenuEvent(self, pos):
    #ContextMenu
    index = self.list_view_.indexAt(pos)
    if index.isValid(): 
      if self.list_model_.IsEditable(index.row()):
        self.contextMenu_.popup(QtWidgets.QCursor.pos())
      else:
        self.immucontextMenu_.popup(QtWidgets.QCursor.pos())
  
  def Add(self):
    if(self.list_view_.currentIndex().isValid()):
      self.list_view_.closePersistentEditor(self.list_view_.currentIndex())
    row = self.list_model_.GetLastRow()
    if self.list_model_.AddItem(datetime.now().isoformat(' '), self.gradient_edit_.GetGradient(), row, True, True):
      index = self.list_model_.index(row)
      self.list_view_.setCurrentIndex(index)
      self.Rename()
    else:
      QtWidgets.QMessageBox.information(self, "Gradient not added", "The gradient could not be added!")
      
  def Remove(self):
    if(self.list_view_.currentIndex().isValid()):
      ret = QtWidgets.QMessageBox.warning(self, "Delete Gradient",
                   "Delete Gradient?",
                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
      if ret == QtWidgets.QMessageBox.Yes:
        self.list_model_.RemoveItem(self.list_view_.currentIndex().row())

  def LoadCurrentIndex(self):
    if(self.list_view_.currentIndex().isValid()):
      self.Load(self.list_view_.currentIndex())

  def Load(self, index):
    if(index.isValid()):
      self.gradientSelected.emit(self.list_model_.GetGradient(index))

  def Rename(self):
    if(self.list_view_.currentIndex().isValid()):
      self.list_view_.edit(self.list_view_.currentIndex())
      
  def GetText(self):
    return self.text_
      



sdh=io.LoadPDB('sdh.pdb')
helix=sdh.Select('rnum=75:78 and cname=A and aname=CA,C,N,O')
res_a=helix.residues[0]
res_b=helix.residues[-1]

def MakePlane(name, pt1, pt2, pt3):
  p1=[pt1, pt2, pt3, pt2+(pt1-pt2)+(pt3-pt2)]  
  c1=sum(p1)/4.0
  p1=[c1+(p-c1)*1.5 for p in p1]
  plane=gfx.Quad(name, *p1)
  plane.SetLineWidth(1.0)
  plane.SetFillColor(gfx.Color(0.3, 0.7, 0.3, 0.5))  
  scene.Add(plane)

r1_n=res_a.FindAtom("N").pos
r1_ca=res_a.FindAtom("CA").pos
r1_c=res_a.FindAtom("C").pos
MakePlane("Plane1", r1_n, r1_ca, r1_c)
r2_ca=res_b.FindAtom("CA").pos
r2_c=res_b.FindAtom("C").pos
r2_o=res_b.FindAtom("O").pos
MakePlane("Plane2", r2_ca, r2_c, r2_o)

ca_dist=gfx.PrimList("Calpha dist")
ca_dist.AddLine(r1_ca, r2_ca, gfx.GREEN)
scene.Add(ca_dist)
go=gfx.Entity('helix2', gfx.SIMPLE, helix)
scene.Add(go)
scene.SetBackground(gfx.WHITE)
go.SetRenderMode(gfx.CUSTOM)
scene.center=go.center


import sys
import unittest
from ost import *
from ost import settings
from ost import seq
from ost.bindings.clustalw import *
from ost.seq.alg import renumber

try:
  clustalw_path=settings.Locate(('clustalw', 'clustalw2'))
except(settings.FileNotFound):
  clustalw_path=None

class TestRenumber(unittest.TestCase):
  
  def setUp(self):
    self.target_seq = io.LoadSequence("testfiles/peptide.fasta")
    self.peptide_original = io.LoadEntity("testfiles/peptide_original.pdb")
    self.peptide_plus_5 = io.LoadEntity("testfiles/peptide_plus_5.pdb")
    self.peptide_random = io.LoadEntity("testfiles/peptide_random.pdb")
    self.peptide_del_1_2 = io.LoadEntity("testfiles/peptide_del_1_2.pdb")
    self.peptide_del_4 = io.LoadEntity("testfiles/peptide_del_4.pdb")
    self.peptide_mutation_3 = io.LoadEntity("testfiles/peptide_mutation_3.pdb")
    
  def testRenumbersChainsBasedOnSequence(self): 
    aln_seq = seq.CreateSequence('A', 'MP-T---NA')
    aln_seq.AttachView(self.peptide_original.Select(''))

    renumbered = renumber.Renumber(aln_seq)

    res = renumbered.residues
    self.assertEqual(len(res), 5)
    self.assertEqual(res[0].number, 1)
    self.assertEqual(res[1].number, 2)
    self.assertEqual(res[2].number, 4)
    self.assertEqual(res[3].number, 8)
    self.assertEqual(res[4].number, 9)

  def testRenumbersChainsBasedOnAlignment(self): 
    aln_seq = seq.CreateSequence('A', 'MP-T---NA')
    aln_seq.AttachView(self.peptide_original.Select(''))
    aln = seq.CreateAlignment(seq.CreateSequence('A', 'MP-T-XXNA'), aln_seq)

    renumbered = renumber.Renumber(aln)

    res = renumbered.residues
    self.assertEqual(len(res), 5)
    self.assertEqual(res[0].number, 1)
    self.assertEqual(res[1].number, 2)
    self.assertEqual(res[2].number, 4)
    self.assertEqual(res[3].number, 8)
    self.assertEqual(res[4].number, 9)

  def testRenumberPreservesBonds(self):
    aln_seq = seq.CreateSequence('A', 'MP-T---NA')
    aln_seq.AttachView(self.peptide_original.Select(''))

    renumbered = renumber.Renumber(aln_seq)

    self.assertTrue(mol.BondExists(renumbered.chains[0].FindAtom(1, 'N'), 
                                   renumbered.chains[0].FindAtom(1, 'CA')))

  def testClustalWPeptidePlusFive(self):
    """
    All residue numbers shifted by 5.
    Check whether internal atom order changes while renumbering
    (a new entity is generated in the edit_mode)
    TODO: add more basic tests: are all properties preserved?
    """
    if not clustalw_path:
      return
    model_seq=seq.SequenceFromChain(" ", self.peptide_plus_5.chains[0]) 
    model_seq.name="model"
    aln=ClustalW(self.target_seq,model_seq)
    aln.AttachView(1,self.peptide_plus_5.Select(""))
    ent_n=renumber.Renumber(aln)

    for r_counter in range(len(self.peptide_original.residues)):
      assert self.peptide_original.residues[r_counter].qualified_name == ent_n.residues[r_counter].qualified_name, \
              "Renumbering failed on residue level: restoring from ResNum+5"
      for atom_nr in range(len(self.peptide_original.residues[r_counter].atoms)):
        assert self.peptide_original.residues[r_counter].atoms[atom_nr].name==ent_n.residues[r_counter].atoms[atom_nr].name, \
                "Renumbering failed on atom level: restoring from ResNum+5"
  

  def testClustalWPeptideRandom(self):
    """
    Change residue names in random order
    """
    if not clustalw_path:
      return
    model_seq=seq.SequenceFromChain(" ", self.peptide_random.chains[0]) 
    model_seq.name="model"
    aln=ClustalW(self.target_seq,model_seq)
    aln.AttachView(1,self.peptide_random.Select(""))
    ent_n=renumber.Renumber(aln)

    for r_counter in range(len(self.peptide_original.residues)):
      assert self.peptide_original.residues[r_counter].qualified_name == ent_n.residues[r_counter].qualified_name, \
             "Renumbering failed on residue level: restoring from random residue numbers"
      for atom_nr in range(len(self.peptide_original.residues[r_counter].atoms)):
        assert self.peptide_original.residues[r_counter].atoms[atom_nr].name==ent_n.residues[r_counter].atoms[atom_nr].name, \
               "Renumbering failed on atom level: restoring from random residue numbers"


  def testClustalWPeptideDel_1_2(self):
    """
    First two residues were removed
    """
    if not clustalw_path:
      return
    model_seq=seq.SequenceFromChain(" ", self.peptide_del_1_2.chains[0]) 
    model_seq.name="model"
    aln=ClustalW(self.target_seq,model_seq)
    aln.AttachView(1,self.peptide_del_1_2.Select(""))
    ent_n=renumber.Renumber(aln)

    for r_counter in range(len(self.peptide_original.residues)):
      if r_counter==0 or r_counter==1:
        continue
      assert self.peptide_original.residues[r_counter].qualified_name == ent_n.residues[r_counter-2].qualified_name, \
             "Renumbering failed on residue level: restoring from random residue numbers"
      for atom_nr in range(len(self.peptide_original.residues[r_counter].atoms)):
        assert self.peptide_original.residues[r_counter].atoms[atom_nr].name==ent_n.residues[r_counter-2].atoms[atom_nr].name, \
               "Renumbering failed on atom level: restoring from random residue numbers"


  def testClustalWPeptideDel_4(self):
    """
    Residues in the middle (position 4) was removed
    """
    if not clustalw_path:
      return
    model_seq=seq.SequenceFromChain(" ", self.peptide_del_4.chains[0]) 
    model_seq.name="model"
    aln=ClustalW(self.target_seq,model_seq)
    aln.AttachView(1,self.peptide_del_4.Select(""))
    ent_n=renumber.Renumber(aln)

    flag=0
    for r_counter in range(len(self.peptide_original.residues)):
      if r_counter==3:
        flag=1
        continue
      assert self.peptide_original.residues[r_counter].qualified_name == ent_n.residues[r_counter-flag].qualified_name, \
             "Renumbering failed on residue level: restoring from random residue numbers"
      for atom_nr in range(len(self.peptide_original.residues[r_counter].atoms)):
        assert self.peptide_original.residues[r_counter].atoms[atom_nr].name==ent_n.residues[r_counter-flag].atoms[atom_nr].name, \
               "Renumbering failed on atom level: restoring from random residue numbers"


  def testClustalWPeptideMutation_3(self):
    """
    Mutation to GLY at postion 3
    """
    if not clustalw_path:
      return
    model_seq=seq.SequenceFromChain(" ", self.peptide_mutation_3.chains[0]) 
    model_seq.name="model"
    aln=ClustalW(self.target_seq,model_seq)
    aln.AttachView(1,self.peptide_mutation_3.Select(""))
    self.assertRaises(RuntimeError, renumber.Renumber, aln)



if __name__ == "__main__":
  # test renumbering
  # test if clustalw package is available on system, otherwise ignore tests
  if not clustalw_path:
    print("Could not find clustalw executable: ignoring some renumber unit tests")
  from ost import testutils
  testutils.RunTests()



from ost.mol import alg
# tabula rasa first
scene.RemoveAll()

# load the fragment
frag=io.LoadPDB('data/1crn.pdb')

# now let's calculate the size of the map by taking the bounding box of
# the fragment and adding 5 Angstrom on each side.
frag_bounds=frag.bounds
real_size=frag_bounds.max-frag_bounds.min+geom.Vec3(10,10,10)
# We use 2 pixels per Angstroem, so we have to multiply the size by 2.
map_size=img.Size(int(real_size.x)*2, int(real_size.y)*2, int(real_size.z)*2)
den_map=img.CreateImage(map_size)
# we use 2 pixels per Angstrom
den_map.SetSpatialSampling(2)
den_map.SetAbsoluteOrigin(frag_bounds.min-geom.Vec3(5, 5, 5))


# Calculate a density map from the fragment using a simple real-space method:
#
# This function is based on Dimaio et al.,J. Mol. Biol(2009)
# In essence every atom gives rise to a Gaussian centered at the atom's
# position. Sigma and Magnitude of the Gaussian are calculated from the
# atomic mass.
alg.EntityToDensityRosetta(frag.CreateFullView(), den_map,
                           alg.HIGH_RESOLUTION, 5.0)

# Create Graphical Representations and add to the Scene
gfx_frag=gfx.Entity('Fragment', gfx.HSC, frag)
gfx_frag.SetColor(gfx.YELLOW)
gfx_map=gfx.MapIso('Density', den_map, 250.0)
scene.Add(gfx_frag)
scene.Add(gfx_map)
scene.CenterOn(gfx_frag)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
"""
Wrappers for the tmalign and tmscore utilities.

References:

tmscore: Yang Zhang and Jeffrey Skolnick, Proteins 2004 57: 702-710 
tmalign: Y. Zhang and J. Skolnick, Nucl. Acids Res. 2005 33, 2302-9 


Authors: Pascal Benkert, Marco Biasini
"""

import subprocess, os, tempfile, platform
import ost
from ost import settings, io, geom, seq

def _SetupFiles(models):
  # create temporary directory
  tmp_dir_name=tempfile.mkdtemp()
  dia = 'PDB'
  for index, model in enumerate(models):
    for chain in model.chains:
      if len(chain.name) > 1:
        dia = 'CHARMM'
        break;
      for res in chain.residues:
        if len(res.name) > 3:
          dia = 'CHARMM'
          break;
    io.SavePDB(model, os.path.join(tmp_dir_name, 'model%02d.pdb' % (index+1)), dialect=dia)
  return tmp_dir_name

def _CleanupFiles(dir_name):
  import shutil
  shutil.rmtree(dir_name)

def _ParseTmAlign(lines,lines_matrix):
  info_line=lines[12].split(',')
  aln_length=int(info_line[0].split('=')[1].strip())
  rmsd=float(info_line[1].split('=')[1].strip())  
  tm_score=float(lines[14].split('=')[1].split('(')[0].strip())
  tf1=[float(i.strip()) for i in lines_matrix[2].split()]
  tf2=[float(i.strip()) for i in lines_matrix[3].split()]
  tf3=[float(i.strip()) for i in lines_matrix[4].split()]
  rot=geom.Mat3(tf1[2], tf1[3], tf1[4], tf2[2], tf2[3],
                tf2[4], tf3[2], tf3[3], tf3[4])
  tf=geom.Mat4(rot)
  tf.PasteTranslation(geom.Vec3(tf1[1], tf2[1], tf3[1]))
  seq1 = seq.CreateSequence("1",lines[18].strip())
  seq2 = seq.CreateSequence("2",lines[20].strip())
  alignment = seq.CreateAlignment()
  alignment.AddSequence(seq2)
  alignment.AddSequence(seq1)
  return ost.bindings.TMAlignResult(rmsd, tm_score, aln_length, tf, alignment)

def _RunTmAlign(tmalign, tmp_dir):
  model1_filename=os.path.join(tmp_dir, 'model01.pdb')
  model2_filename=os.path.join(tmp_dir, 'model02.pdb')
  if platform.system() == "Windows":
    tmalign_path=settings.Locate('tmalign.exe', explicit_file_name=tmalign)
    command="\"%s\" %s %s -m %s" %(os.path.normpath(tmalign_path), model1_filename, model2_filename, os.path.join(tmp_dir,'matrix.txt'))
  else:
    tmalign_path=settings.Locate('tmalign', explicit_file_name=tmalign)  
    command="\"%s\" \"%s\" \"%s\" -m \"%s\"" %(tmalign_path, model1_filename, model2_filename, os.path.join(tmp_dir,'matrix.txt'))
  ps=subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
  stdout,_=ps.communicate()
  lines=stdout.decode().splitlines()
  if (len(lines))<22:
    _CleanupFiles(tmp_dir)
    raise RuntimeError("tmalign superposition failed")
  matrix_file=open(os.path.join(tmp_dir,'matrix.txt'))
  lines_matrix=matrix_file.readlines()
  matrix_file.close() 
  return _ParseTmAlign(lines,lines_matrix)


class TMScoreResult:
  """
  Holds the result of running TMscore
  
  .. attribute:: rmsd_common
    
    The RMSD of the common Calpha atoms of both structures

  .. attribute:: rmsd_below_five

    The RMSD of all Calpha atoms that can be superposed below five Angstroem
    
  .. attribute:: tm_score
  
    The TM-score of the structural superposition
  
  .. attribute:: transform
  
    The transform that superposes the model onto the reference structure.
    
    :type: :class:`~ost.geom.Mat4`
  
  .. attribute:: gdt_ha
  
    The GDT_HA of the model to the reference structure.

  .. attribute:: gdt_ts

    The GDT_TS of the model to the reference structure.

  """
  def __init__(self, rmsd_common, tm_score, max_sub, 
               gdt_ts, gdt_ha, rmsd_below_five, transform):
    self.rmsd_common=rmsd_common
    self.tm_score=tm_score    
    self.max_sub=max_sub
    self.gdt_ts=gdt_ts
    self.gdt_ha=gdt_ha
    self.rmsd_below_five=rmsd_below_five
    self.transform=transform
    
def _ParseTmScore(lines):
  tf1=[float(i.strip()) for i in lines[23].split()]
  tf2=[float(i.strip()) for i in lines[24].split()]
  tf3=[float(i.strip()) for i in lines[25].split()]
  rot=geom.Mat3(tf1[2], tf1[3], tf1[4], tf2[2], tf2[3],
                  tf2[4], tf3[2], tf3[3], tf3[4])
  tf=geom.Mat4(rot)
  tf.PasteTranslation(geom.Vec3(tf1[1], tf2[1], tf3[1]))
  result=TMScoreResult(float(lines[14].split()[-1].strip()),
                       float(lines[16].split()[2].strip()),
                       float(lines[17].split()[1].strip()),
                       float(lines[18].split()[1].strip()),
                       float(lines[19].split()[1].strip()),
                       float(lines[27].split()[-1].strip()),
                       tf)
  return result

def _RunTmScore(tmscore, tmp_dir):
  model1_filename=os.path.join(tmp_dir, 'model01.pdb')
  model2_filename=os.path.join(tmp_dir, 'model02.pdb')  
  if platform.system() == "Windows":
    tmscore_path=settings.Locate('tmscore.exe', explicit_file_name=tmscore)
    command="\"%s\" %s %s" %(os.path.normpath(tmscore_path), model1_filename, 
                             model2_filename)
  else:
    tmscore_path=settings.Locate('tmscore', explicit_file_name=tmscore)
    command="\"%s\" \"%s\" \"%s\"" % (tmscore_path, model1_filename, 
                                      model2_filename)
  ps=subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
  stdout,_=ps.communicate()
  lines=stdout.decode().splitlines()
  if (len(lines))<22:
    _CleanupFiles(tmp_dir)
    raise RuntimeError("tmscore superposition failed")
  return _ParseTmScore(lines)


def TMAlign(model1, model2, tmalign=None):
  """
  Performs a sequence independent superposition of model1 onto model2, the 
  reference.
  

  :param model1: The model structure. If the superposition is successful, will 
                 be superposed onto the reference structure
  :type model1: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param model2: The reference structure
  :type model2: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param tmalign: If not None, the path to the tmalign executable.
  :returns: The result of the tmscore superposition
  :rtype: :class:`ost.bindings.TMAlignResult`
  
  :raises: :class:`~ost.settings.FileNotFound` if tmalign could not be located.
  :raises: :class:`RuntimeError` if the superposition failed
  """
  tmp_dir_name=_SetupFiles((model1, model2))
  result=_RunTmAlign(tmalign, tmp_dir_name)
  model1.handle.EditXCS().ApplyTransform(result.transform)
  _CleanupFiles(tmp_dir_name)
  return result

def TMScore(model1, model2, tmscore=None):
  """
  Performs a sequence dependent superposition of model1 onto model2, 
  the reference.

  :param model1: The model structure. If the superposition is successful, will 
                 be superposed onto the reference structure
  :type model1: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param model2: The reference structure
  :type model2: :class:`~ost.mol.EntityView` or :class:`~ost.mol.EntityHandle`
  :param tmscore: If not None, the path to the tmscore executable.
  :returns: The result of the tmscore superposition
  :rtype: :class:`TMScoreResult`
  
  :raises: :class:`~ost.settings.FileNotFound` if tmalign could not be located.
  :raises: :class:`RuntimeError` if the superposition failed
  """
  tmp_dir_name=_SetupFiles((model1, model2))
  result=_RunTmScore(tmscore, tmp_dir_name)
  model1.handle.EditXCS().ApplyTransform(result.transform)  
  _CleanupFiles(tmp_dir_name)
  return result
